"""SECS/GEM Functions.

The following functions allow you to interact with equipment defined by
the SECS/GEM module. Note that the SECS/GEM module must be installed
before these functions will be accessible.
"""

from __future__ import print_function

__all__ = [
    "copyEquipment",
    "deleteToolProgram",
    "enableDisableEquipment",
    "getResponse",
    "getToolProgram",
    "getToolProgramDataset",
    "sendRequest",
    "sendResponse",
    "startSimEventRun",
    "toDataSet",
    "toTreeDataSet",
]

import system.date
from com.inductiveautomation.ignition.common import BasicDataset, Dataset


def copyEquipment(
    equipmentSource,
    newEquipmentName,
    enabled,
    activeAddress,
    activePort,
    passiveAddress,
    passivePort,
    deviceId,
    dbTablePrefix=None,
    description=None,
):
    """Creates a copy of an equipment connection.

    Common settings can be overridden for the new connection.

    An exception is thrown if the new Equipment Connection cannot be
    created.

    Args:
        equipmentSource (str): Some new equipment settings will be
            retrieved from this equipment connection. Specify the source
            equipment connection name.
        newEquipmentName (str): The name of the new equipment
            connection.
        enabled (bool): If set to False, the new equipment connection
            will be disabled after it is created.
        activeAddress (str): IP Address of new equipment. Must be
            specified if the SECS/GEM module is used in Active mode.
            Otherwise, do not use this parameter.
        activePort (int): Port number of new equipment. Must be
            specified if the SECS/GEM module is used in Active mode.
            Otherwise, do not use this parameter.
        passiveAddress (str): IP Address of new equipment. Must be
            specified if the SECS/GEM module is used in Passive mode.
            Otherwise, do not use this parameter.
        passivePort (int): Port number of new equipment. Must be
            specified if the SECS/GEM module is used in Passive mode.
            Otherwise, do not use this parameter.
        deviceId (int): Unique identifier of new equipment. This value
            must be an integer, and is specified within the equipment.
        dbTablePrefix (str): SECS/GEM database table names will use the
            specified prefix for the new equipment connection. If no
            prefix is specified, the description of the source equipment
            will be used. Optional.
        description (str): The description for the new equipment
            connection. If no description is specified, the description
            of the source equipment will be used. Optional.
    """
    print(
        equipmentSource,
        newEquipmentName,
        enabled,
        activeAddress,
        activePort,
        passiveAddress,
        passivePort,
        deviceId,
        dbTablePrefix,
        description,
    )


def deleteToolProgram(ppid):
    """Deletes a process program from the Gateway.

    Args:
        ppid (str): The PPID that was sent from the tool when the S7F3
            message was saved.
    """
    print(ppid)


def enableDisableEquipment(enable, names):
    """Enables or disables a Tuple of equipment connections from a
    script.

    Args:
        enable (bool): Set to True to enable equipment connections, or
            set to False to disable them.
        names (tuple[str]): A Tuple of Strings. Each String should match
            an Equipment Connection configured on the Gateway. If this
            parameter contains the name of an Equipment Connection that
            does not exist, then a message will be included in the List
            returned by this function.

    Returns:
         list[str]: A List of unicode strings. Each string contains a
            message about an equipment connection that could not be
            enabled/disabled. If the list is empty, then all specified
            equipment connections were successfully modified.
    """
    print(enable)
    for name in names:
        print(name)
    return [""]


def getResponse(transactionID, equipment, timeout=5, poll=150):
    """Attempts to retrieve a response message from the Gateway.

    The transaction id from the sent message is used to retrieve the
    response.

    Args:
        transactionID (int): The transactionID of the request and
            response. The transactionID is used to retrieve the
            response. Typically used in conjunction with
            system.secsgem.sendRequest to generate a transactionID.
        equipment (str): Name of equipment connection.
        timeout (int): Specifies in seconds how long to wait for a
            response before returning None. If omitted the timeout will
            be 5 seconds. Optional.
        poll (int): Specifies in milliseconds how often to poll the
            system for a response. If omitted the poll will be 150
            milliseconds. Optional.

    Returns:
        object: A Python object, typically a dictionary. The actual
            result is a JSON string that's decoded into a python object,
            as shown on the mapping on the system.util.jsonDecode page.
    """
    print(transactionID, equipment, timeout, poll)


def getToolProgram(ppid):
    """Returns a process program from the Gateway that was previously
    sent by a a tool in an S7F3 message.

    Args:
        ppid (str): The PPID that was sent from the tool when the S7F3
            message was saved.

    Returns:
        dict: A Python Dictionary containing the following keys:
            [editDate, ppbody, bodyFormat].
    """
    print(ppid)
    return {
        "editDate": system.date.now(),
        "ppBody": "program",
        "bodyFormat": "A",
    }


def getToolProgramDataset():
    """Returns a Dataset containing information about all stored process
    programs.

    Returns:
         BasicDataset: A Dataset containing information about all stored
            process programs. Includes the following columns in order:
            ppid, editDate, bodyFormat.
    """
    return BasicDataset()


def sendRequest(streamFunction, reply, body, equipment):
    """Sends a JSON-formatted SECS message to a tool.

    An equipment connection must be configured for the tool in the
    Gateway.

    Args:
        streamFunction (str): The stream and function of the SECS
            message to send. Example: "S1F13"
        reply (bool): Whether or not the SECS message expects a reply
            message.
        body (object): This contains the body of a SECS message. The
            argument can be a Python Object or JSON string representing
            the body of a SECS message. If this argument is a string
            then it will be converted to a Python Object using the
            system.util.jsonDecode function.
        equipment (str): Name of the equipment connection to use.

    Returns:
        int: The transactionID of the SECS message response.
    """
    print(streamFunction, reply, body, equipment)
    return 0


def sendResponse(transactionID, systemBytes, streamFunction, body, equipment):
    """Sends a JSON-formatted SECS response message to a message sent by
    a tool.

    An equipment connection must be configured for the tool in the
    Gateway, and this must be used within a Message Handler to create a
    Custom Message Response Handler.

    Args:
        transactionID (int): The TxID of the response. The TxID from the
            received request in the payload of the message handler must
            be specified here.
        systemBytes (int): The SystemBytes of the response. The
            SystemBytes from the received request in the payload of the
            message handler must be specified here.
        streamFunction (str): The stream and function of the SECS
            message to send. Example: "S1F14".
        body (object): This contains the body of a SECS response. The
            argument can be a Python Object or JSON string representing
            the body of a SECS message. If this argument is a string
            then it will be converted to a Python Object using the
            system.util.jsonDecode function.
        equipment (str): Name of the equipment connection to use.
    """
    print(transactionID, systemBytes, streamFunction, body, equipment)


def startSimEventRun(simulatorName, eventRunName):
    """Starts a configured simulator event run in the Gateway.

    Note, that this function only works with the simulators that come
    included with the SECS/GEM module.

    The function will throw an exception if the specified Event Run
    cannot be started.

    Args:
        simulatorName (str): The simulator that holds the configured
            event run. Will throw an exception if the specified
            simulator can't be found.
        eventRunName (str): The event run to start. Will throw an
            exception if the specified simulator can't be found.
    """
    print(simulatorName, eventRunName)


def toDataSet(secsObject):
    """Converts a SECS message data structure, as returned by the
    system.secsgem.getResponse function, into a dataset and returns it.

    Args:
        secsObject (object): A Python object, such as Sequence or a
            Dictionary, representing a SECS message to convert to a
            dataset.

    Returns:
         BasicDataset: A Dataset representing a SECS message.
    """
    print(secsObject)
    return BasicDataset()


def toTreeDataSet(dataset):
    """Changes an existing dataset, as returned by the
    system.secsgem.toDataSet function, to make it usable for the Tree
    View component.

    Args:
        dataset (Dataset): A DataSet containing a SECS message. Note
            that this parameter cannot take a JSON message, so the
            object returned by system.secsgem.getResponse must first be
            processed by system.secsgem.toDataSet.

    Returns:
        BasicDataset: A Dataset containing a SECS message that can be
            used in the Tree View component.
    """
    print(dataset)
    return BasicDataset()



"""Package java.awt.

Contains all of the classes for creating user interfaces and for
painting graphics and images.
"""

from __future__ import print_function

__all__ = [
    "Color",
    "Component",
    "Container",
    "Frame",
    "Image",
    "Toolkit",
    "Window",
]

from java.lang import Object


class Color(Object):
    """The Color class is used to encapsulate colors in the default sRGB
    color space or colors in arbitrary color spaces identified by a
    ColorSpace.

    Every color has an implicit alpha value of 1.0 or an
    explicit one provided in the constructor. The alpha value defines
    the transparency of a color and can be represented by a float value
    in the range 0.0 - 1.0 or 0 - 255. An alpha value of 1.0 or 255
    means that the color is completely opaque and an alpha value of 0 or
    0.0 means that the color is completely transparent. When
    constructing a Color with an explicit alpha or getting the
    color/alpha components of a Color, the color components are never
    premultiplied by the alpha component.

    The default color space for the Java 2D(tm) API is sRGB, a proposed
    standard RGB color space. For further information on sRGB, see
    http://www.w3.org/pub/WWW/Graphics/Color/sRGB.html.
    """

    def __init__(self, *args):
        """Creates a color.

        Args:
            *args: Variable length argument list.
        """
        print(args)


class Component(Object):
    """A component is an object having a graphical representation that
    can be displayed on the screen and that can interact with the user.
    """

    pass


class Container(Component):
    """A generic Abstract Window Toolkit(AWT) container object is a
    component that can contain other AWT components.
    """

    def add(self, *args):
        """Adds the specified component.

        Args:
            *args: Variable length argument list.
        """
        print(self, args)


class Image(Object):
    """The abstract class Image is the superclass of all classes that
    represent graphical images.

    The image must be obtained in a platform-specific manner.
    """

    pass


class Toolkit(Object):
    """This class is the abstract superclass of all actual
    implementations of the Abstract Window Toolkit.

    Subclasses of the Toolkit class are used to bind the various
    components to particular native toolkit implementations.
    """

    def beep(self):
        """Emits an audio beep depending on native system settings and
        hardware capabilities.
        """
        print(self, "Beep!")

    @staticmethod
    def getDefaultToolkit():
        """Gets the default toolkit."""
        return Toolkit()


class Window(Container):
    """A Window object is a top-level window with no borders and no
    menubar.
    """

    pass


class Frame(Window):
    """A Frame is a top-level window with a title and a border."""

    pass



"""Device Functions.

The following functions give you access to view and edit device
connections in the Gateway.
"""

from __future__ import print_function

__all__ = [
    "addDevice",
    "getDeviceHostname",
    "listDevices",
    "refreshBrowse",
    "removeDevice",
    "setDeviceEnabled",
    "setDeviceHostname",
]

from com.inductiveautomation.ignition.common import BasicDataset


def addDevice(deviceType, deviceName, deviceProps, description=None):
    """Adds a new device connection in Ignition.

    Accepts a dictionary of parameters to configure the connection.
    Acceptable parameters differ by device type: i.e., a Modbus/TCP
    connection requires a hostname and port, but a simulator doesn't
    require any parameters.

    Args:
        deviceType (str): The device driver type. Possible values are
            listed in the Device Types table below.
        deviceName (str): The name that will be given to the the new
            device connection.
        deviceProps (dict): A dictionary of device connection properties
            and values. Each deviceType has different properties, but
            most require at least a hostname. Keys in the dictionary are
            case-insensitive, spaces are omitted, and the names of the
            properties that appear when manually creating a device
            connection.
        description (str): The description that will be given to the
            new device connection. Optional.
    """
    print(deviceType, deviceName, deviceProps, description)


def getDeviceHostname(deviceName):
    """Gets the hostname of a device.

    Args:
        deviceName (str): The name of the device in Ignition.

    Returns:
        str: The hostname of the device. Null if device doesn't have a
            hostname.
    """
    print(deviceName)


def listDevices():
    """Returns a dataset of information about each configured device.

    Each row represents a single device.

    Returns:
        BasicDataset: A dataset, where each row represents a device.
            Contains 4 columns Name, Enabled, State, and Driver.
    """
    return BasicDataset()


def refreshBrowse(deviceName):
    """Forces Ignition to browse the controller.

    Only works for Allen-Bradley controllers.

    Args:
        deviceName (str): The name of the device in Ignition.
    """
    print(deviceName)


def removeDevice(deviceName):
    """Removes a given device from Ignition.

    Args:
        deviceName (str): The name of the device in Ignition.
    """
    print(deviceName)


def setDeviceEnabled(deviceName, enabled):
    """Enables/disables a device in Ignition.

    Args:
        deviceName (str): The name of the device in Ignition.
        enabled (bool): The enabled status of the device.
    """
    print(deviceName, enabled)


def setDeviceHostname(deviceName, hostname):
    """Changes the hostname of a device.

    Used for all ethernet based drivers.

    Args:
        deviceName (str): The name of the device in Ignition.
        hostname (str): The new IP address or hostname.
    """
    print(deviceName, hostname)



__all__ = ["Version"]

import re

from java.lang import Object


class Version(Object):
    dev = True
    snapshot = False

    def __init__(self, major=0, minor=0, rev=0, build=0, beta=0, rc=0):
        """Version initializer.

        Args:
            major (int): Major number.
            minor (int): Minor number.
            rev (int): Revision number.
            build (int): Build number.
            beta (int): Beta number.
            rc (int): Release Candidate number.
        """
        self.major = major
        self.minor = minor
        self.rev = rev
        self.build = build
        self.beta = beta
        self.rc = rc

    def __eq__(self, other, strict=False):
        ret = 0
        version_1 = [self.major, self.minor, self.rev]
        version_2 = [other.major, other.minor, other.rev]

        if strict:
            version_1.append(self.build)
            version_1.append(self.beta)
            version_1.append(self.rc)
            version_2.append(other.build)
            version_2.append(other.beta)
            version_2.append(other.rc)

        for i in range(max(len(version_1), len(version_2))):
            v_1 = version_1[i] if i < len(version_1) else 0
            v_2 = version_2[i] if i < len(version_2) else 0
            if v_1 > v_2:
                ret = 1
            elif v_1 < v_2:
                ret = -1
        return ret

    def __str__(self):
        return self.toString()

    def compareTo(self, that):
        """Compares two Versions.

        Note that this comparison is stricter than we want for Gateway
        restores or project imports. For those, isFutureVersion().

        Args:
            that (Version): The version to compare.

        Returns:
             int: 0 if self and that are equal, -1 if that is greater
                than self, or 1 if self is greater than that.
        """
        return self.__eq__(that, True)

    def exists(self):
        pass

    @staticmethod
    def fromXML(inputStream):
        pass

    def getBasicString(self):
        if self.rc > 0:
            ret = "{}.{}.{}-rc{}".format(
                self.major, self.minor, self.rev, self.rc
            )
        else:
            ret = "{}.{}.{}".format(self.major, self.minor, self.rev)
        return ret

    def getBeta(self):
        return self.beta

    def getBuildNumber(self):
        return self.build

    def getMajor(self):
        return self.major

    def getMinor(self):
        return self.minor

    def getRc(self):
        return self.rc

    def getRevision(self):
        return self.rev

    def getXML(self):
        pass

    def isDev(self):
        return self.dev

    def isFutureVersion(self, arg):
        other = self.parse(arg)
        return self.__eq__(other) == -1

    def isSnapshot(self):
        return self.snapshot

    @staticmethod
    def parse(s):
        sem_ver = [int(i) for i in re.findall(r"-?\d+", s)]
        return Version(sem_ver[0], sem_ver[1], sem_ver[2])

    def toParseableString(self):
        """Returns the version as a compact, parseable (non-XML) string
        that can be parsed with the 1-string constructor of this class.

        Returns:
            string: Compact, parseable (non-XML) string.
        """
        return "{}.{}.{}.{}".format(
            self.major, self.minor, self.rev, self.build
        )

    def toString(self):
        if self.rc > 0:
            version = "{}.{}.{}-rc{} (b{})".format(
                self.major, self.minor, self.rev, self.rc, self.build
            )
        elif self.isSnapshot():
            version = "{}.{}.{}-SNAPSHOT (b{})".format(
                self.major, self.minor, self.rev, self.build
            )
        elif self.build is not None:
            version = "{}.{}.{} (b{})".format(
                self.major, self.minor, self.rev, self.build
            )
        else:
            version = "{}.{}.{}".format(self.major, self.minor, self.rev)
        return version



"""Perspective Functions.

The following functions offer various ways to interact with a
Perspective session from a Python script.
"""

from __future__ import print_function

__all__ = [
    "alterLogging",
    "closeDock",
    "closePage",
    "closePopup",
    "closeSession",
    "download",
    "getProjectInfo",
    "getSessionInfo",
    "isAuthorized",
    "login",
    "logout",
    "navigate",
    "navigateBack",
    "navigateForward",
    "openDock",
    "openPopup",
    "print",
    "refresh",
    "sendMessage",
    "setTheme",
    "toggleDock",
    "togglePopup",
    "vibrateDevice",
]

try:
    import __builtin__ as builtins
except ImportError:
    import builtins

from com.inductiveautomation.ignition.common.script.adapters import (
    PyJsonObjectAdapter,
)


def alterLogging(
    remoteLoggingEnabled=False,
    level="info",
    remoteLoggingLevel="warn",
    sessionId="current_session",
    pageId="current_page",
):
    """Changes Perspective Session logging attributes and levels.

    All parameters are optional, with the caveat that at least one of
    them needs to be used.

    Args:
        remoteLoggingEnabled (bool): Will enable remote logging if True.
            Remote logging will send log events from the Session to the
            Gateway under the perspective.client logger if the meet the
            remoteLevel logging level requirement. Optional.
        level (str): The desired Session logging level. Possible values
            are: all, trace, debug, info, warn, error, fatal, off. The
            default is info. Optional.
        remoteLoggingLevel (str): The desired remote logging level.
            Possible values are: all, trace, debug, info, warn, error,
            fatal, off. The default is warn. Optional.
        sessionId (str): Identifier of the Session to target. If
            omitted, the current Session will be used automatically.
            When targeting a different session, then the pageId
            parameter must be included in the call. Optional.
        pageId (str): Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
    """
    builtins.print(
        remoteLoggingEnabled, level, remoteLoggingLevel, sessionId, pageId
    )


def closeDock(id, sessionId=None, pageId=None):
    """Closes a docked view.

    Args:
        id (str): The unique, preconfigured 'Dock ID' for the docked
            View. Is specified when a View is assigned as docked for a
            particular Page (in Page Configuration).
        sessionId (str): Identifier of the Session to target. If
            omitted, the current Session will be used automatically.
            When targeting a different session, then the pageId
            parameter must be included in the call. Optional.
        pageId (str): Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
    """
    builtins.print(id, sessionId, pageId)


def closePage(
    message=None, sessionId="current_session", pageID="current_page"
):
    """Closes the page with the given page id or the current page if no
    page id is provided.

    If a message is provided, it is displayed on the page when the page
    closes. Otherwise the default message (set in the Project
    Properties) is displayed.

    Args:
        message (str): The message to display when the page closes. If
            omitted, the default message (set in the Project Properties)
            is shown. Optional.
        sessionId (str): Identifier of the Session to target. If
            omitted, the current Session will be used automatically.
            When targeting a different session, then the pageId
            parameter must be included in the call. Optional.
        pageID (str): Identifier of the page to be closed. If omitted,
            the current pageId is used. Optional.
    """
    builtins.print(message, sessionId, pageID)


def closePopup(id, sessionId="current_session", pageId="current_page"):
    """Closes a popup View.

    Args:
        id (str): The unique identifier for the the popup, given to the
            popup when first opened. If given an empty string, then the
            most recently focused popup will be closed.
        sessionId (str): Identifier of the Session to target. If
            omitted, the current Session will be used automatically.
            When targeting a different session, then the pageId
            parameter must be included in the call. Optional.
        pageId (str): Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
    """
    builtins.print(id, sessionId, pageId)


def closeSession(message=None, sessionId="current_session"):
    """Closes the Perspective Session with the given session ID or the
    current session if no ID is provided.

    If a message is provided, it is displayed on the page when the
    session closes. Otherwise the default message (set in the Project
    Properties) is displayed.

    Args:
        message (str): The message to display when the session closes.
            If omitted, the default message (set in the Project
            Properties) is shown. Optional.
        sessionId (str): Identifier of the session to be closed. If
            omitted, the current sessionId is used. Optional.
    """
    builtins.print(message, sessionId)


def download(
    filename,
    data,
    contentType=None,
    sessionId="current_session",
    pageId="current_page",
):
    """Downloads data from the gateway to a device running a session.

    Args:
        filename (str): Suggested name for the downloaded file.
        data (object): The data to be downloaded. May be a String, a
            byte[], or an InputStream. Strings will be written with in
            "utf-8" encoding.
        contentType (str): Value for the "Content-Type" header. Example:
            "text/plain; charset=utf-8". Optional.
        sessionId (str): Identifier of the Session to target. If omitted
            the current Session will be used automatically. When
            targeting a different session, then the pageId parameter
            must be included in the call. Optional.
        pageId (str): Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
    """
    builtins.print(filename, data, contentType, sessionId, pageId)


def getProjectInfo(projectName):
    """Returns a dictionary of meta data from a Perspective Project.

    Args:
        projectName (str): The name of the project.

    Returns:
        dict: A dictionary of project meta data.
    """
    builtins.print(projectName)
    return {
        "name": "Project",
        "title": "Project",
        "description": "Project's description",
        "lastModified": "2021-04-01T13:37:00.000Z",
        "lastModifiedBy": "thecesrom",
        "views": [{"path": "Page/Home"}],
    }


def getSessionInfo(usernameFilter=None, projectFilter=None):
    """Returns information about one or more Perspective sessions.

    The information returned by this function is a combination of
    information available on the perspective sessions status page on the
    Gateway, and some session props (id and userAgent).

    Args:
        usernameFilter (str): A filter based on logged in user.
            Optional.
        projectFilter (str): A filter based on the project name.
            Optional.

    Returns:
        list[PyJsonObjectAdapter]: A list of objects
            (PyJsonObjectAdapter).
    """
    builtins.print(usernameFilter, projectFilter)
    return [PyJsonObjectAdapter(None)]


def isAuthorized(isAllOf, securityLevels, sessionId="current_session"):
    """Checks if the user in the current session is authorized against a
    target collection of security levels.

    Args:
        isAllOf (bool): True if the current user must have all of the
            given security levels to be authorized. False if the current
            user must have at least one of the given security levels to
            be authorized.
        securityLevels (list[str]): An array of string paths to a
            security level node in the form of "Path/To/Node". Each
            level in the tree is delimited by a forward slash character.
            The public node is never a part of the path.
        sessionId (str): Identifier of the Session to target. If
            omitted, the current Session will be used automatically.
            Optional.

    Returns:
        bool: True if the user in the current session is authorized,
            False otherwise.
    """
    builtins.print(isAllOf, securityLevels, sessionId)
    return True


def login(sessionId="current_session", pageId="current_page", forceAuth=False):
    """Triggers a login event that will allow the user to login with the
    project's configured Identity Provider (IdP).

    For this function to work, an Identity Provider must be set in
    Perspective project properties. This is particularly useful when you
    want it to be possible to start a session without authentication and
    sign in to access certain restricted features.

    Args:
        sessionId (str): Identifier of the Session to target. If omitted
            the current Session will be used automatically. When
            targeting a different session, then the pageId parameter
            must be included in the call. Optional.
        pageId (str): Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
        forceAuth (bool): Determines if Ignition should ask the Identity
            Provider to re-authenticate the user, even if the user is
            already signed into the Identity Provider. If set to True,
            then the Identity Provider will ask the user to re-enter
            their credentials. If set to False, then the Gateway will
            request that the Identity Provider use the last provided
            credentials for the session, potentially allowing
            re-authentication without requiring the user to re-type
            their credentials. Note that support for this argument is
            determined by the Identity Provider; the IdP may choose to
            ignore this request. If this parameter is omitted, then the
            function will use the re-authentication setting defined
            under Project Properties. Optional.
    """
    builtins.print(sessionId, pageId, forceAuth)


def logout(sessionId="current_session", pageId="current_page"):
    """Triggers a logout event, which will log the user out.

    For this function to work, an Identity Provider must be set in the
    Perspective project properties.

    Args:
        sessionId (str): Identifier of the Session to target. If omitted
            the current Session will be used automatically. When
            targeting a different session, then the pageId parameter
            must be included in the call. Optional.
        pageId (str): Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
    """
    builtins.print(sessionId, pageId)


def navigate(
    page,
    url=None,
    view=None,
    params=None,
    sessionId="current_session",
    pageId="current_page",
    newTab=False,
):
    """Navigate the session to a specified view or mounted page.

    The function can be used in three different ways, depending on which
    parameter is specified:
        page: navigates to a perspective-page
        url: navigates to a web address, so the function can be used to
            navigate the user to a web portal, search engine, or other
            website. This parameter is specified via keyword argument
        view: navigates to a view. Note that using this parameter does
            not modify the web browser's address bar, so the browser's
            history will not contain a listing for the new view. This
            parameter is specified via keyword argument

    Args:
        page (str): The URL of a Perspective page to navigate to.
        url (str): The URL of a web address to navigate to. If the page
            or view parameters are specified, then this parameter is
            ignored.
        view (str): If specified, will navigate to a specific view.
            Navigating to a view via this parameter does not change the
            address in the web browser. Thus the web browser's back
            button will not be able to return the user to the previous
            view. If the page parameter is specified, then this
            parameter is ignored. Optional.
        params (dict): Used only in conjunction with the view parameter,
            Dictionary of values to pass to any parameters on the view.
            Optional.
        sessionId (str): Identifier of the Session to target. If omitted
            the current Session will be used automatically. When
            targeting a different session, then the pageId parameter
            must be included in the call. Optional.
        pageId (str): Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
        newTab (bool): If True, opens the contents in a new tab.
            Optional.
    """
    builtins.print(page, url, view, params, sessionId, pageId, newTab)


def navigateBack(sessionId=None, pageId=None):
    """Navigate the session to a specified view or mounted page.

    This is similar to a browser's "back" function.

    Args:
        sessionId (str): Identifier of the Session to target. If
            omitted, the current Session will be used automatically.
            Optional.
        pageId (str): Identifier of the page to target. If omitted, the
            current page will be used automatically. Optional.
    """
    builtins.print(sessionId, pageId)


def navigateForward(sessionId=None, pageId=None):
    """Navigate the session to a specified view or mounted page.

    This is similar to a browser's "forward" function.

    Args:
        sessionId (str): Identifier of the Session to target. If
            omitted, the current Session will be used automatically.
            When targeting a different session, then the pageId
            parameter must be included in the call. Optional.
        pageId (str): Identifier of the page to target. If omitted, the
            current page will be used automatically. Optional.
    """
    builtins.print(sessionId, pageId)


def openDock(
    id, sessionId="current_session", pageId="current_page", params=None
):
    """Opens a docked View.

    Requires the preconfigured dock ID for the view.

    Args:
        id (str): The unique, preconfigured 'Dock ID' for the docked
            View. Is specified when a View is assigned as docked for a
            particular Page (in Page Configuration).
        sessionId (str): Identifier of the Session to target. If omitted
            the current Session will be used automatically. When
            targeting a different session, then the pageId parameter
            must be included in the call. Optional.
        pageId (str): Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
        params (dict): Parameters that can be passed into the docked
            view. Must match the docked views View Parameters. Optional.
    """
    builtins.print(id, sessionId, pageId, params)


def openPopup(
    id,
    view,
    params=None,
    title="",
    position=None,
    showCloseIcon=True,
    draggable=True,
    resizable=False,
    modal=False,
    overlayDismiss=False,
    sessionId="current_session",
    pageId="current_page",
    viewPortBound=False,
):
    """Open a popup view over the given page.

    Args:
        id (str): A unique popup string. Will be used to close the popup
            from other popup or script actions.
        view (str): The path to the View to use in the popup.
        params (dict): Dictionary of key-value pairs to us as input
            parameters to the View. Optional.
        title (str): Text to display in the title bar. If no value or an
            empty string are given, the title bar will not be displayed.
            Defaults to an empty string. Optional.
        position (dict): Dictionary of key-value pairs to use for
            position. Possible position keys are: left, top, right,
            bottom, width, height. Defaults to the center of the window.
            Optional.
        showCloseIcon (bool): Will show the close icon if True. Defaults
            to True. Optional.
        draggable (bool): Will allow the popup to be dragged if True.
            Defaults to True. Optional.
        resizable (bool): Will allow the popup to be resized if True.
            Defaults to False. Optional.
        modal (bool): Will make the popup modal if True. A modal popup
            is the only view the user can interact with. Defaults to
            False. Optional.
        overlayDismiss (bool): Will allow the user to dismiss and close
            a modal popup by clicking outside of it if True. Defaults to
            False. Optional.
        sessionId (str): Identifier of the Session to target. If omitted
            the current Session will be used automatically. When
            targeting a different session, then the pageId parameter
            must be included in the call. Optional.
        pageId (str): Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
        viewPortBound (bool): If True, popups will be "shifted" to open
            within the bounds of the viewport. If the popup would be
            larger than the viewport, then it will be resized to fit
            within the bounds. Default is False. Optional.
    """
    builtins.print(
        id,
        view,
        params,
        title,
        position,
        showCloseIcon,
        draggable,
        resizable,
        modal,
        overlayDismiss,
        sessionId,
        pageId,
        viewPortBound,
    )


def print(
    message,
    sessionId="current_session",
    pageId="current_page",
    destination="client",
):
    """Sends print statements to the scripting console when in the
    Designer.

    When in a Session, sends print statements to the output console.
    This function makes scripting diagnostics easier.

    Args:
        message (str): The print statement that will be displayed on the
            console.
        sessionId (str): Identifier of the Session to target. If omitted
            the current Session will be used automatically. When
            targeting a different session, then the pageId parameter
            must be included in the call. Optional.
        pageId (str): Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
        destination (str): Where the message should be printed. If
            specified, must be "client", "gateway", or "all". Default is
            "client". Optional.
    """
    builtins.print(message, sessionId, pageId, destination)


def refresh(sessionId="current_session", pageId="current_page"):
    """Triggers a refresh of the page.

    Args:
        sessionId (str): Identifier of the Session to target. If
            omitted, the current Session will be used automatically.
            When targeting a different session, then the pageId
            parameter must be included in the call. Optional.
        pageId (str): Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
    """
    builtins.print(sessionId, pageId)


def sendMessage(
    messageType,
    payload,
    scope="page",
    sessionId="current_session",
    pageId="current_page",
):
    """Send a message to a message handler within the same session.

    Args:
        messageType (str): The message type that will be invoked.
            Message handlers configured within the project are listening
            for messages of a specific messageType.
        payload (dict): A python dictionary representing any parameters
            that will be passed to the message handler.
        scope (str): The scope that the message should be delivered to.
            Valid values are "session", "page", or "view". If omitted,
            "page" will be used. Optional.
        sessionId (str): Identifier of the Session to target. If
            omitted, the current Session will be used automatically.
            When targeting a different session, then the pageId
            parameter must be included in the call. Optional.
        pageId (str): Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
    """
    builtins.print(messageType, payload, scope, sessionId, pageId)


def setTheme(name, sessionId="current_session", pageId="current_page"):
    """Changes the theme in a page to the specified theme.

    Note that this function only changes the theme for a single page,
    not the entire session. To change the theme for a session, write
    directly to the session.theme property instead.

    Args:
        name (str): The theme name to switch to. Possible values are
            "dark" or "light".
        sessionId (str): Identifier of the Session to target. If
            omitted the current Session will be used automatically.
            When targeting a different session, then the pageId
            parameter must be included in the call. Optional.
        pageId (str): Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
    """
    builtins.print(name, sessionId, pageId)


def toggleDock(
    id, sessionId="current_session", pageId="current_page", params=None
):
    """Toggles a docked View.

    Args:
        id (str): The unique, preconfigured 'Dock ID' for the docked
            View. Is specified when a View is assigned as docked for a
            particular Page (in Page Configuration).
        sessionId (str): Identifier of the Session to target. If
            omitted the current Session will be used automatically.
            When targeting a different session, then the pageId
            parameter must be included in the call. Optional.
        pageId (str): Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
        params (dict): Parameters that can be passed into the docked
            view. Must match the docked views View Parameters. Optional.
    """
    builtins.print(id, sessionId, pageId, params)


def togglePopup(
    id,
    view,
    params,
    title="",
    position=None,
    showCloseIcon=True,
    draggable=True,
    resizable=False,
    modal=False,
    overlayDismiss=False,
    sessionId="current_session",
    pageId="current_page",
    viewPortBound=False,
):
    """Toggles a popup.

    Will open up the popup if it has not been opened yet. Otherwise, it
    will close the currently opened popup.

    Args:
        id (str): A unique popup string. Will be used to close the popup
            from other popup or script actions.
        view (str): The path to the View to use in the popup.
        params (dict): Dictionary of key-value pairs to us as input
            parameters to the View. Optional.
        title (str): Text to display in the title bar. If no value or an
            empty string are given, the title bar will not be displayed.
            Defaults to an empty string. Optional.
        position (dict): Dictionary of key-value pairs to use for
            position. Possible position keys are: left, top, right,
            bottom, width, height. Defaults to the center of the window.
            Optional.
        showCloseIcon (bool): Will show the close icon if True. Defaults
            to True. Optional.
        draggable (bool): Will allow the popup to be dragged if True.
            Defaults to True. Optional.
        resizable (bool): Will allow the popup to be resized if True.
            Defaults to False. Optional.
        modal (bool): Will make the popup modal if True. A modal popup
            is the only view the user can interact with. Defaults to
            False. Optional.
        overlayDismiss (bool): Will allow the user to dismiss and close
            a modal popup by clicking outside of it if True. Defaults to
            False. Optional.
        sessionId (str): Identifier of the Session to target. If omitted
            the current Session will be used automatically. When
            targeting a different session, then the pageId parameter
            must be included in the call. Optional.
        pageId (str): Identifier of the Page to target. If omitted, the
            current Page will be used automatically. Optional.
        viewPortBound (bool): If True, popups will be "shifted" to open
            within the bounds of the viewport. If the popup would be
            larger than the viewport, then it will be resized to fit
            within the bounds. Default is False. Optional.
    """
    builtins.print(
        id,
        view,
        params,
        title,
        position,
        showCloseIcon,
        draggable,
        resizable,
        modal,
        overlayDismiss,
        sessionId,
        pageId,
        viewPortBound,
    )


def vibrateDevice(duration, sessionId="current_session"):
    """When called from the Perspective mobile app, will cause the
    device to vibrate for the specified number of milliseconds.

    Args:
        duration (int): The duration in milliseconds to vibrate the
            device.
        sessionId (str): Identifier of the Session to target. If
            omitted the current Session will be used automatically. When
            targeting a different session, then the pageId parameter
            must be included in the call. Optional.
    """
    builtins.print(duration, sessionId)



"""Perspective Workstation Functions.

The following functions offer various ways to interact with Perspective
Workstation from a Python script. Note that the functions here only work
when called from a session running in Perspective Workstation.
"""

__all__ = ["exit", "toKiosk", "toWindowed"]


def exit():
    """When called from a session running in Workstation, this function
    will close Workstation.
    """
    pass


def toKiosk():
    """When called from a session running in Perspective Workstation,
    attempts to put Workstation into Kiosk mode.
    """
    pass


def toWindowed():
    """When called from a Session running in Perspective Workstation,
    attempts to put Workstation into windowed mode.
    """
    pass



"""DNP3 Functions.

The following functions give you access to interact with the DNP3
devices.
"""

from __future__ import print_function

__all__ = [
    "directOperateAnalog",
    "directOperateBinary",
    "freezeAnalogs",
    "freezeAnalogsAtTime",
    "freezeCounters",
    "freezeCountersAtTime",
    "selectOperateAnalog",
    "selectOperateBinary",
]

# Constants
NUL = 0
PULSE_ON = 1
PULSE_OFF = 2
LATCH_ON = 3
LATCH_OFF = 4
CLOSE = 1
TRIP = 2


def directOperateAnalog(deviceName, index, value, variation=None):
    """Issues a Select-And-Operate command to set an analog value in an
    analog output point.

    Args:
        deviceName (str): The name of the DNP3 device driver.
        index (int): The index of the object to be modified in the
            outstation.
        value (object): The analog value that is requested (of type int,
            short, float, or double).
        variation (int): The DNP3 object variation to use in the
            request. Optional.

    Returns:
        int: The DNP3 status code of the response, as an integer.
    """
    print(deviceName, index, value, variation)
    return 0


def directOperateBinary(
    deviceName,
    indexes,
    opType,
    tcCode=None,
    count=None,
    onTime=None,
    offTime=None,
):
    """Issues a Direct-Operate command for digital control operations at
    binary output points (CROB).

    Args:
        deviceName (str): The name of the DNP3 device driver.
        indexes (list[object]): A list of indexes of the objects to be
            modified in the outstation.
        opType (int): The type of the operation. 0=NUL, 1=PULSE_ON,
            2=PULSE_OFF, 3=LATCH_ON, 4=LATCH_OFF.
        tcCode (int): The Trip-Close code, used in conjunction with the
            opType. 0=NUL, 1=CLOSE, 2=TRIP. Optional.
        count (int): The number of times the outstation shall execute
            the operation. Optional.
        onTime (long): The duration that the output drive remains
            active, in millis. Optional.
        offTime (long): The duration that the output drive remains
            non-active, in millis. Optional.

    Returns:
        int: The DNP3 status code of the response, as an integer.
    """
    print(deviceName, indexes, opType, tcCode, count, onTime, offTime)
    return 0


def freezeAnalogs(deviceName, indexes=None):
    """Issues a freeze command on the given analog outputs.

    Args:
        deviceName (str): The name of the DNP3 device driver.
        indexes (list[object]): An optional list of specific indexes on
            which to issue the freeze command.
    """
    print(deviceName, indexes)


def freezeAnalogsAtTime(deviceName, absoluteTime, intervalTime, indexes=None):
    """Issues a freeze command on the given analog outputs at the given
    time for the specified duration.

    Args:
        deviceName (str): The name of the DNP3 device driver.
        absoluteTime (int): The absolute time at which to freeze, in
            millis.
        intervalTime (int): The interval at which to periodically
            freeze, in millis.
        indexes (list[object]): An optional list of specific indexes on
            which to issue the freeze command. Optional.
    """
    print(deviceName, absoluteTime, intervalTime, indexes)


def freezeCounters(deviceName, indexes=None):
    """Issues a freeze command on the given counters.

    Args:
        deviceName (str): The name of the DNP3 device driver.
        indexes (list[object]): An optional list of specific indexes on
            which to issue the freeze command. Optional.
    """
    print(deviceName, indexes)


def freezeCountersAtTime(deviceName, absoluteTime, intervalTime, indexes=None):
    """Issues a freeze command on the given counters at the given time
    for the specified duration.

    Args:
        deviceName (str): The name of the DNP3 device driver.
        absoluteTime (int): The absolute time at which to freeze, in
            millis.
        intervalTime (int): The interval at which to periodically
            freeze, in millis.
        indexes (list[object]): An optional list of specific indexes on
            which to issue the freeze command. Optional.
    """
    print(deviceName, absoluteTime, intervalTime, indexes)


def selectOperateAnalog(deviceName, index, value, variation=None):
    """Issues a Select-And-Operate command to set an analog value in an
    analog output point.

    Args:
        deviceName (str): The name of the DNP3 device driver.
        index (int): The index of the object to be modified in the
            outstation.
        value (object): The analog value that is requested (of type int,
            short, float, or double).
        variation (int): The DNP3 object variation to use in the
            request. Optional.

    Returns:
        int: The DNP3 status code of the response, as an integer.
    """
    print(deviceName, index, value, variation)


def selectOperateBinary(
    deviceName,
    indexes,
    opType,
    tcCode=None,
    count=None,
    onTime=None,
    offTime=None,
):
    """Issues a Select-And-Operate command for digital control
    operations at binary output points (CROB).

    Args:
        deviceName: The name of the DNP3 device driver.
        indexes (list[object]): A list of indexes of the objects to be
            modified in the outstation.
        opType (int): The type of operation. 0=NUL, 1=PULSE_ON,
            2=PULSE_OFF, 3=LATCH_ON, 4=LATCH_OFF
        tcCode (int): The Trip-Close code, used in conjunction with the
            opType. 0=NUL, 1=CLOSE, 2=TRIP. Optional.
        count (int): The number of times the outstation shall execute
            the operation. Optional.
        onTime (int): The duration that the output drive remains active,
            in millis. Optional.
        offTime (int): The duration that the output drive remains
            non-active, in millis. Optional.

    Returns:
        int: The DNP3 status code of the response, as an integer.
    """
    print(deviceName, indexes, opType, tcCode, count, onTime, offTime)
    return 0



"""Provides for system input and output through data streams,
serialization and the file system. Unless otherwise noted, passing a
null argument to a constructor or method in any class or interface in
this package will cause a NullPointerException to be thrown.
"""

from __future__ import print_function

__all__ = [
    "DataOutputStream",
    "FileDescriptor",
    "FileOutputStream",
    "FilterOutputStream",
    "OutputStream",
    "PrintStream",
]

from java.lang import Object


class FileDescriptor(Object):
    def sync(self):
        pass

    def valid(self):
        pass


class OutputStream(Object):
    def close(self):
        pass

    def flush(self):
        pass

    @staticmethod
    def nullOutputStream():
        return OutputStream()

    def write(self, *args):
        pass


class FileOutputStream(OutputStream):
    def __init__(self, *args):
        pass

    def getChannel(self):
        pass

    def getFD(self):
        print(self)
        return FileDescriptor()


class FilterOutputStream(OutputStream):
    _out = OutputStream()

    def __init__(self, out):
        self._out = out


class DataOutputStream(FilterOutputStream):
    _written = 0

    def __init__(self, out):
        self.out = out
        super(DataOutputStream, self).__init__(out)

    def size(self):
        pass

    def writeBoolean(self, v):
        pass

    def writeByte(self, v):
        pass

    def writeBytes(self, s):
        pass

    def writeChar(self, v):
        pass

    def writeChars(self, s):
        pass

    def writeDouble(self, v):
        pass

    def writeFloat(self, v):
        pass

    def writeInt(self, v):
        pass

    def writeLong(self, v):
        pass

    def writeShort(self, v):
        pass

    def writeUTF(self, s):
        pass


class PrintStream(FilterOutputStream):
    _out = OutputStream()

    def __init__(self, *args):
        print(args)
        super(PrintStream, self).__init__(self._out)

    def append(self, *args):
        pass

    def checkError(self):
        pass

    def clearError(self):
        pass

    def format(self, *args):
        pass

    def print(self, arg):
        pass

    def printf(self, *args):
        pass

    def println(self, arg):
        pass

    def setError(self):
        pass



"""Contains the collections framework, legacy collection classes, event
model, date and time facilities, internationalization, and miscellaneous
utility classes (a string tokenizer, a random-number generator, and a
bit array).
"""

__all__ = ["Date", "EventObject", "Locale"]

from datetime import datetime

from java.lang import Object


class Date(datetime):
    """The class Date represents a specific instant in time, with
    millisecond precision.
    """

    def __new__(cls, date=None):
        """Allocates a Date object and initializes it...

        1) so that it represents the time at which it was allocated,
        measured to the nearest millisecond.

        2) to represent the specified number of milliseconds since the
        standard base time known as "the epoch", namely January 1, 1970,
        00:00:00 GMT.

        1) java.util.Date()
        2) java.util.Date(date)

        Args:
            date (long): The milliseconds since January 1, 1970,
                00:00:00 GMT. Optional.

        Returns:
            Date: A new Date instance.
        """
        return datetime.now() if date is None else Date._from_millis(date)

    @staticmethod
    def _from_millis(millis):
        seconds = millis // 1000
        micro = (millis % 1000) * 1000
        date = datetime.fromtimestamp(seconds)
        return datetime(
            date.year,
            date.month,
            date.day,
            date.hour,
            date.minute,
            date.second,
            micro,
        )


class EventObject(Object):
    """The root class from which all event state objects shall be
    derived.

    All Events are constructed with a reference to the object, the
    "source", that is logically deemed to be the object upon which the
    Event in question initially occurred upon.
    """

    def __init__(self, source):
        """Constructs a prototypical Event.

        Args:
            source (Object): The object on which the Event initially
                occurred.
        """
        self.source = source

    def getSource(self):
        """The object on which the Event initially occurred."""
        return self.source


class Locale(Object):
    """A Locale object represents a specific geographical, political, or
    cultural region. An operation that requires a Locale to perform its
    task is called locale-sensitive and uses the Locale to tailor
    information for the user. For example, displaying a number is a
    locale-sensitive operation; the number should be formatted according
    to the customs and conventions of the user's native country, region,
    or culture.
    """

    def __init__(self, language, country=None, variant=None):
        """Locale initializer.

        Args:
            language (str): Language code.
            country (str): Country code.
            variant (str): Variant code.
        """
        self.language = language
        self.country = country
        self.variant = variant

    @property
    def CANADA(self):
        """Useful constant for country."""
        return self.__init__("en", "CA")

    @property
    def CANADA_FRENCH(self):
        """Useful constant for country."""
        return self.__init__("fr", "CA")

    @property
    def CHINA(self):
        """Useful constant for country."""
        return self.__init__("zh", "CN")

    @property
    def CHINESE(self):
        """Useful constant for country."""
        return self.__init__("zh")

    @property
    def ENGLISH(self):
        """Useful constant for country."""
        return self.__init__("en")

    @property
    def FRANCE(self):
        """Useful constant for country."""
        return self.__init__("fr", "FR")

    @property
    def FRENCH(self):
        """Useful constant for country."""
        return self.__init__("fr")

    @property
    def GERMAN(self):
        """Useful constant for country."""
        return self.__init__("de")

    @property
    def GERMANY(self):
        """Useful constant for country."""
        return self.__init__("de", "DE")

    @property
    def ITALIAN(self):
        """Useful constant for country."""
        return self.__init__("it")

    @property
    def ITALY(self):
        """Useful constant for country."""
        return self.__init__("it", "IT")

    @property
    def JAPAN(self):
        """Useful constant for country."""
        return self.__init__("ja", "JP")

    @property
    def JAPANESE(self):
        """Useful constant for country."""
        return self.__init__("ja")

    @property
    def KOREA(self):
        """Useful constant for country."""
        return self.__init__("ko", "KR")

    @property
    def KOREAN(self):
        """Useful constant for country."""
        return self.__init__("ko")

    @property
    def PRC(self):
        """Useful constant for country."""
        return self.CHINA

    @property
    def SIMPLIFIED_CHINESE(self):
        """Useful constant for country."""
        return self.CHINA

    @property
    def TAIWAN(self):
        """Useful constant for country."""
        return self.__init__("zh", "TW")

    @property
    def TRADITIONAL_CHINESE(self):
        """Useful constant for country."""
        return self.__init__("zh", "TW")

    @property
    def UK(self):
        """Useful constant for country."""
        return self.__init__("en", "GB")

    @property
    def US(self):
        """Useful constant for country."""
        return self.__init__("en", "US")



"""OPC HDA Functions.

The following functions give you access to interact with the HDA types
of OPC servers.
"""

from __future__ import print_function

__all__ = [
    "browse",
    "getAggregates",
    "getAttributes",
    "getServers",
    "insert",
    "insertReplace",
    "isServerAvailable",
    "readAttributes",
    "readProcessed",
    "readRaw",
    "replace",
]

from com.inductiveautomation.ignition.common.browsing import Results
from com.inductiveautomation.ignition.common.sqltags.history import (
    Aggregate,
    AggregateInfo,
)
from java.util import Date


def browse(root):
    """Performs a browse at the given root.

    Args:
        root (str): The root at which to browse. Needs to be a qualified
            path.

    Returns:
        list[Results]: The Browse Results that would result for the
            operation at that root. Refer to the list of Results
            objects.
    """
    print(root)
    return [Results()]


def getAggregates(serverName):
    """Will query the server for aggregates that it supports.

    Args:
        serverName (str): The name of the defined OPC-HDA server to
            query.

    Returns:
        list[Aggregate]: A list of supported Aggregate objects. Each
            object has 'id', 'name', and 'desc' properties defined.
    """
    print(serverName)
    return [AggregateInfo()]


def getAttributes(serverName):
    """Queries the given server for the item attributes that are
    available with system.opchda.readAttributes().

    Args:
        serverName (str): The name of the defined OPC-HDA server to
            query.

    Returns:
        list[Aggregate]: A list of supported Aggregate objects. Each
            object has 'id', 'name', and 'desc' properties defined.
    """
    print(serverName)
    return [AggregateInfo()]


def getServers():
    """Returns a list of the OPC-HDA servers configured on the system.

    This call will return all configured and enabled servers, including
    those that are not currently connected.

    Returns:
        list[str]: A list of the string names of servers.
    """
    return []


def insert(serverName, itemId, value, date, quality):
    """Insert values on the OPC-HDA server if the given item ID does not
    exist.

    Args:
        serverName (str): The name of the defined OPC-HDA server.
        itemId (str): The item ID to perform the operation on.
        value (object): The value to insert.
        date (object): The date to insert.
        quality (int): The quality to insert.

    Returns:
        int: The items quality form the operation.
    """
    print(serverName, itemId, value, date, quality)
    return 192


def insertReplace(serverName, itemId, value, date, quality):
    """Will insert values on the OPC-HDA server, or replace them if they
    already exist.

    Args:
        serverName (str): The name of the defined OPC-HDA server.
        itemId (str): The item ID to perform the operation on.
        value (object): The value to insert.
        date (object): The date to insert.
        quality (int): The quality to insert.

    Returns:
        int: The items quality form the operation.
    """
    print(serverName, itemId, value, date, quality)
    return 192


def isServerAvailable(serverName):
    """Checks to see if the specified OPC-HDA server is defined,
    enabled, and connected.

    Args:
        serverName (str): The name of the OPC-HDA server to check.

    Returns:
        bool: Will be True if the server is available and can be
            queried, False if not.
    """
    print(serverName)
    return True


def readAttributes(serverName, itemId, attributeIds, startDate, endDate):
    """Reads the specified attributes for the given item over a time
    range.

    Attributes and their IDs are defined in the OPC-HDA specification,
    and can be discovered by calling system.opchda.getAttributes().

    Args:
        serverName (str): The name of the defined OPC-HDA server to
            read.
        itemId (str): The itemID to retrieve attributes for.
        attributeIds (list[int]): The integer IDs of the attributes to
            read. The attribute ids are defined in the OPC-HDA
            specification. The attributes can also be obtained by
            calling system.opchda.getAttributes(). Some servers may not
            support all attributes.
        startDate (Date): The starting date/time of the query.
        endDate (Date): The ending date/time of the query.

    Returns:
        list[ReadResult]: A list of read results which is one-to-one
            with the requested attributes. The ReadResult object has a
            'serviceResult' quality property that indicates whether the
            call was successful, and is itself a list of
            QualifiedValues.
    """
    print(serverName, itemId, attributeIds, startDate, endDate)
    return []


def readProcessed(
    serverName, itemIds, startDate, endDate, resampleIntervalMS, aggregates
):
    """Reads processed values from the OPC-HDA server.

    Processed values are calculated values, based on the aggregate
    function requested for each item. The list of aggregates can be
    obtained by calling system.opchda.getAggregates().

    Args:
        serverName (str): The name of the defined OPC-HDA server to
            read.
        itemIds (list[str]): A list of item ids to read.
        startDate (Date): The starting date/time of the query.
        endDate (Date): The ending date/time of the query.
        resampleIntervalMS (int): The interval, in milliseconds, that
            each value should cover.
        aggregates (list[object]): A list which should be one-to-one
            with the item ids requested, specifying the integer id of
            the aggregation function to use. The aggregation ids are
            defined in the OPC-HDA specification. The list of aggregates
            can also be obtained by calling
            system.opchda.getAggregates().

    Returns:
        list[ReadResult]: A list of read results which is one-to-one
            with the item IDs passed in. The ReadResult object has a
            'serviceResult' quality property that indicates whether the
            call was successful, and is itself a list of
            QualifiedValues.
    """
    print(
        serverName,
        itemIds,
        startDate,
        endDate,
        resampleIntervalMS,
        aggregates,
    )
    return []


def readRaw(
    serverName, itemIds, startDate, endDate, maxValues, boundingValues
):
    """Reads raw values from the OPC-HDA server.

    Args:
        serverName (str): The name of the defined OPC-HDA server to
            read.
        itemIds (list[str]): A list of item ids to read.
        startDate (Date): The starting date/time of the query.
        endDate (Date): The ending date/time of the query.
        maxValues (int): The maximum number of values to return. 0 or
            less means unlimited.
        boundingValues (bool): A boolean indicating whether or not the
            "bounding values" should be included in the result set. The
            bounding values provide a value exactly at the start and end
            dates, but may be resource-intensive to retrieve.

    Returns:
        list[ReadResult]: A list of read results which is one-to-one
            with the item IDs passed in. The ReadResult object has a
            'serviceResult' quality property that indicates whether the
            call was successful, and is itself a list of
            QualifiedValues.
    """
    print(serverName, itemIds, startDate, endDate, maxValues, boundingValues)
    return []


def replace(serverName, itemId, value, date, quality):
    """Replaces values on the OPC-HDA server if the given item ID
    exists.

    Args:
        serverName (str): The name of the defined OPC-HDA server.
        itemId (str): The item ID to perform the operation on.
        value (object): The value to replace.
        date (Date): The date to replace.
        quality (int): The quality to replace.

    Returns:
        int: The item's quality resulting from the operation.
    """
    print(serverName, itemId, value, date, quality)
    return 192



__all__ = ["AbstractDataset", "BasicDataset", "Dataset"]


class Dataset(object):
    """A dataset is a collection of values arranged in a structured
    format.

    Most datasets are two dimensional -- they can be viewed as a table
    with rows and columns being the two dimensions. Values in a dataset
    are usually accessed by specifying one index for each dimension of
    data (row and column for tables).
    """

    def binarySearch(self, column, key):
        pass

    def getColumnAsList(self, col):
        pass

    def getColumnCount(self):
        raise NotImplementedError

    def getColumnIndex(self, name):
        raise NotImplementedError

    def getColumnName(self, col):
        raise NotImplementedError

    def getColumnNames(self):
        raise NotImplementedError

    def getColumnType(self, col):
        raise NotImplementedError

    def getColumnTypes(self):
        raise NotImplementedError

    def getPrimitiveValueAt(self, row, col):
        raise NotImplementedError

    def getQualityAt(self, row, col):
        raise NotImplementedError

    def getRowCount(self):
        raise NotImplementedError

    def getValueAt(self, row, col):
        raise NotImplementedError

    def hasQualityData(self):
        pass


class AbstractDataset(Dataset):
    _columnNames = None
    _columnNamesLowercase = None
    _columnTypes = None
    _qualityCodes = None

    def __init__(self, columnNames, columnTypes, qualityCodes=None):
        self._columnNames = columnNames
        self._columnTypes = columnTypes
        self._qualityCodes = qualityCodes

    @staticmethod
    def convertToQualityCodes(dataQualities):
        pass

    def getBulkQualityCodes(self):
        pass

    def getColumnCount(self):
        pass

    def getColumnIndex(self, name):
        pass

    def getColumnName(self, col):
        pass

    def getColumnNames(self):
        pass

    def getColumnType(self, col):
        pass

    def getColumnTypes(self):
        pass

    def getPrimitiveValueAt(self, row, col):
        pass

    def getQualityAt(self, row, col):
        pass

    def getRowCount(self):
        pass

    def getValueAt(self, row, col):
        pass

    def setColumnNames(self, arg):
        pass

    def setColumnTypes(self, arg):
        pass

    def setDirty(self):
        pass


class BasicDataset(AbstractDataset):
    def __init__(self, columnNames=None, columnTypes=None):
        super(BasicDataset, self).__init__(columnNames, columnTypes)

    def columnContainsNulls(self, col):
        pass

    def datasetContainsNulls(self):
        pass

    def getData(self):
        pass

    def setAllDirectly(self, columnNames, columnTypes, data):
        pass

    def setDataDirectly(self, arg):
        pass

    def setFromXML(self, columnNames, columnTypes, encodedData, rowCount):
        pass

    def setValueAt(self, row, col, value):
        pass



"""Tag Functions.

The following functions give you access to interact with Ignition Tags.
"""

from __future__ import print_function

__all__ = [
    "browse",
    "browseHistoricalTags",
    "browseTags",
    "browseTagsSimple",
    "configure",
    "copy",
    "deleteAnnotations",
    "deleteTags",
    "exists",
    "exportTags",
    "getConfiguration",
    "importTags",
    "isOverlaysEnabled",
    "move",
    "queryAnnotations",
    "queryTagCalculations",
    "queryTagDensity",
    "queryTagHistory",
    "read",
    "readAll",
    "readAsync",
    "readBlocking",
    "rename",
    "requestGroupExecution",
    "setOverlaysEnabled",
    "storeAnnotations",
    "storeTagHistory",
    "write",
    "writeAll",
    "writeAsync",
    "writeBlocking",
]

import warnings

import system.date
from com.inductiveautomation.ignition.common import BasicDataset
from com.inductiveautomation.ignition.common.browsing import Results
from com.inductiveautomation.ignition.common.model.values import (
    BasicQualifiedValue,
    QualifiedValue,
    QualityCode,
)
from com.inductiveautomation.ignition.common.script.builtin.ialabs import (
    BrowseTag,
)
from java.util import Date


def browse(path, filter=None):
    """Returns a list of tags found at the specified tag path.

    The list objects are returned as dictionaries with some basic
    information about each tag.

    Args:
        path (str): The path that will be browsed, typically to a folder
            or UDT instance.
        filter (dict): A dictionary of browse filter parameters.
            Parameters include name (with wildcards), dataType,
            valueSource, tagType, typeId, quality (specify Good or Bad),
            and maxResults. The aforementioned wildcard character is the
            * character. Again, the wildcard is only used with the Name
            parameter.

    Returns:
        Results: A Results object which contains a list of tag
            dictionaries, one for each tag found during the browse. Use
            .getResults() on the results object to get the list of tag
            dictionaries, or .getReturnedSize() to the the number of
            items in results. Refer to the list of tagBrowse objects.
    """
    print(path, filter)
    return Results()


def browseHistoricalTags(
    path, nameFilters=None, maxSize=None, continuationPoint=None
):
    """Will browse for any historical Tags at the provided historical
    path.

    It will only browse for Tags at the path, and will not go down
    through any children. Will return with a BrowseResults object.

    Args:
        path (str): The Historical Tag Path to browse. See the Tag
            Export page for a description of how to construct a
            historical Tag Path.
        nameFilters (list[str]): A list of name filters to be applied to
            the result set. Optional.
        maxSize (int): The maximum size of the result set. Optional.
        continuationPoint (object): Sets the continuation point in order
            to continue a browse that was previously started and then
            limited. Use .getContinuationPoint() on the BrowseResults
            object to get the continuation point. Optional.

    Returns:
        Results: A Results object which contains a list of Tag
            dictionaries, one for each Tag found during the browse.
    """
    print(path, nameFilters, maxSize, continuationPoint)
    return Results()


def browseTags(
    parentPath,
    tagPath=None,
    tagType=None,
    dataType=None,
    udtParentType=None,
    recursive=False,
    sort="ASC",
):
    """Returns an array of tags from a specific folder.

    The function supports filtering and recursion. Leave filters blank
    to return all tags.

    If called in the gateway scope, a Tag Provider must be specified.

    Args:
        parentPath (str): The parent folder path. Leave blank for the
            root folder. Note: you can specify the tag provider name in
            square brackets at the beginning of the parentPath string.
            Example: "[myTagProvider]MyTagsFolder". If the tag provider
            name is left off then the project default provider will be
            used.
        tagPath (str): Filters on a tag path. Use * as a wildcard for
            any number of characters and a ? for a single character.
            Optional.
        tagType (str): Filters on a tag type. Possible values are OPC,
            MEMORY, DB, QUERY, Folder, DERIVED and UDT_INST. Optional.
        dataType (str): The data type of the tag. Not used for UDT
            instances or folders. Possible values are Int1, Int2, Int4,
            Int8, Float4, Float8, Boolean, String, and DateTime.
            Optional.
        udtParentType (str): The name of the parent UDT. Optional.
        recursive (bool): Recursively search for tags inside of folders.
            Note: It is highly recommended that recursive is set to
            False, as server timeouts are more likely to occur.
            Optional.
        sort (str): Sets the sort order, possible values are ASC and
            DESC. Sorting is done on the full path of the tag. Optional.

    Returns:
        list[BrowseTag]: An array of BrowseTag. BrowseTag has the
            following variables: name, path, fullPath, type, dataType,
            and the following functions: isFolder(), isUDT(), isOPC(),
            isMemory(), isExpression(), isQuery().
    """
    warnings.warn(
        "browseTags is deprecated, use browse instead.", DeprecationWarning
    )
    print(
        parentPath,
        tagPath,
        tagType,
        dataType,
        udtParentType,
        recursive,
        sort,
    )
    return [BrowseTag()]


def browseTagsSimple(parentPath, sort):
    """Returns a sorted array of tags from a specific folder.

    Args:
        parentPath (str): The parent folder path. Leave blank for the
            root folder. Note: you can specify the tag provider name in
            square brackets at the beginning of the parentPath string.
            Example: "[myTagProvider]MyTagsFolder". If the tag provider
            name is left off then the project default provider will be
            used.
        sort (str): Sets the sort order, possible values are ASC and
            DESC.

    Returns:
        list[BrowseTag]: An array of BrowseTag. BrowseTag has the
            following variables: name, path, fullPath, type, dataType,
            and the following functions: isFolder(), isUDT(), isOPC(),
            isMemory(), isExpression(), isQuery().
    """
    warnings.warn(
        "browseTagsSimple is deprecated, use browse instead.",
        DeprecationWarning,
    )
    print(parentPath, sort)
    return [BrowseTag()]


def configure(basePath, tags, collisionPolicy="o"):
    """Creates Tags from a given list of Python dictionaries or from a
    JSON source string.

    Can be used to overwrite a current Tag's configuration.

    When utilizing this function, the tag definitions must specify the
    names of properties with their scripting/JSON name. A reference of
    these properties can be found on the Tag Properties and Tag Alarm
    Properties pages.

    Args:
        basePath (str): The starting point where the new Tags will be
            created. When making changes to existing tags with this
            function, you want to set the path to the parent folder of
            the exiting tag(s), not the tag(s) themselves.
        tags (list[object]): A list of Tag definitions, where each Tag
            definition is a Python dictionary. Alternately, a JSON
            source string may be passed to this parameter. When editing
            existing tags, it is generally easier to retrieve the tag
            configurations with system.tag.getConfiguration, modify the
            results of the getConfiguration call, and then write the new
            configuration to the parent folder of the existing tag(s).
        collisionPolicy (str): The action to take when a tag or folder
            with the same path and name is encountered. Possible values
            include:

            a - Abort and throw an exception
            o - Overwrite and replace existing Tag's configuration
            i - Ignore that item in the list
            m - Merge, modifying values that are specified in the
                definition, without impacting values that aren't defined
                in the definition. Use this when you want to apply a
                slight change to tags, without having to build a
                complete configuration object.

            Defaults to Overwrite. Optional.

    Returns:
        list[QualityCode]: A List of QualityCode objects, one for each
            tag in the list, that is representative of the result of the
            operation.
    """
    print(basePath, tags, collisionPolicy)
    return [QualityCode()]


def copy(tags, destination, collisionPolicy="o"):
    """Copies tags from one folder to another.

    Multiple tag and folder paths may be passed to a single call of this
    function. The new destination can be a separate tag provider.

    Args:
        tags (list[str]): A List of Tag paths to move.
        destination (str): The destination to copy the Tags to. All
            specified tags will be copied to the same destination. The
            destination tag provider must be specified.
        collisionPolicy (str): The action to take when a tag or folder
            with the same path and name is encountered. Possible values
            include: "a" Abort and throw an exception, "o" Overwrite and
            replace existing Tag's configuration, "i" Ignore that item
            in the list. Defaults to Overwrite. Optional.

    Returns:
        list[QualityCode]: A List of QualityCode objects, one for each
            tag in the list, that is representative of the result of the
            operation.
    """
    print(tags, destination, collisionPolicy)
    return [QualityCode()]


def deleteAnnotations(paths, storageIds):
    """Removes stored annotations from the sqlth_annotations table.

    Requires the full tag path (including history provider) for each
    annotation, as well as each annotation's storage ID. Storage ID
    values can be retrieved with system.tag.queryAnnotations.

    Args:
        paths (list[str]): A list of tag paths with existing
            annotations. The paths are equivalent to what would be used
            for a tag history query, and should specify the source
            provider as well. For example,
            "[HistoryProvider/Gateway:Provider]Path/To/Tag".
        storageIds (list[int]): A sequence of storage identifiers for If
            defined, these will be used to perform updates, or deletes
            (if the corresponding delete parameter is True). Storage id
            is available on the Annotation object, and is returned as
            the result value from the storeAnnotations call.

    Returns:
        list[QualifiedValue]: A list of qualified values. The quality
            code will indicate success or failure, and if successful,
            the storage id of the annotation will have been deleted.
    """
    print(paths, storageIds)


def deleteTags(tagPaths):
    """Deletes multiple Tags or Tag Folders.

    When deleting a Tag Folder, all Tags under the folder are also
    deleted.

    Args:
        tagPaths (list[str]): A List of the paths to the Tags or Tag
            Folders that are to be removed.

    Returns:
         list[QualityCode]: A List of QualityCode objects, one for each
            tag in the list, that is representative of the result of the
            operation, e.g. "Good", "Bad_NotFound", etc. The quality
            codes have a built-in isNotGood() method that can be used to
            determine if any deletions failed.
    """
    print(tagPaths)


def exists(tagPath):
    """Checks whether or not a tag with a given path exists.

    Args:
        tagPath (str): The path of the tag to look up.

    Returns:
        bool: True if a tag exists for the given path, False otherwise.
    """
    print(tagPath)
    return True


def exportTags(filePath, tagPaths, recursive=True, exportType="json"):
    """Exports Tags to a file on a local file system.

    The term "local file system" refers to the scope in which the script
    was running; for example, running this script in a Gateway Timer
    script will export the file to the Gateway file system.

    Args:
        filePath (str): The file path that the Tags will be exported to.
            If the file does not already exist, this function will
            attempt to create it.
        tagPaths (list[str]): A List of Tag paths to export. All Tag
            paths in the list must be from the same parent folder.
        recursive (bool): Set to True to export all Tags under each Tag
            path, including Tags in child folders. Defaults to True.
            Optional.
        exportType (str): The type of file that will be exported. Set to
            "json" or "xml". Defaults to "json". Optional.
    """
    print(filePath, tagPaths, recursive, exportType)


def getConfiguration(basePath, recursive=False):
    """Retrieves Tags from the Gateway as Python dictionaries.

    These can be edited and then saved back using system.tag.configure.

    Args:
        basePath (str): The starting point where the Tags will be
            retrieved. This can be a folder containing, and if recursive
            is True, then the function will attempt to retrieve all of
            the tags in the folder.
        recursive (bool): If True, the entire Tag Tree under the
            specified path will be retrieved. Note that this will only
            check one level under the base path. True recursion would
            require multiple uses of this function at different paths.
            Optional.

    Returns:
         dict: A List of Tag dictionaries. Nested Tags are placed in a
            list marked as "tags" in the dictionary.
    """
    print(basePath, recursive)


def importTags(filePath, basePath, collisionPolicy="o"):
    """Imports a JSON tag file at the provided path.

    Also supports XML and CSV Tag file exports from legacy systems.

    Args:
        filePath (str): The file path of the Tag export to import.
        basePath (str): The Tag path that will serve as the root node
            for the imported Tags.
        collisionPolicy (str): The action to take when a tag or folder
            with the same path and name is encountered. Possible values
            include: "a" Abort and throw an exception, "o" Overwrite and
            replace existing Tag's configuration, "i" Ignore that item
            in the list. Defaults to Overwrite. Optional.

    Returns:
        list[QualityCode]: A List of QualityCode objects, one for each
            tag in the list, that is representative of the result of the
            operation.
    """
    print(filePath, basePath, collisionPolicy)
    return [QualityCode()]


def isOverlaysEnabled():
    """Returns whether or not the current client's quality overlay
    system is currently enabled.

    Returns:
         bool: True (1) if overlays are currently enabled.
    """
    return False


def move(tags, destination, collisionPolicy="o"):
    """Moves Tags or Folders to a new destination.

    The new destination can be a separate tag provider. If interested in
    copying the tags to a new destination, instead of moving them,
    please see the system.tag.copy page.

    Args:
        tags (list[str]): A List of Tag paths to move.
        destination (str): The destination to move the Tags to. The
            destination tag provider must be specified: i.e.,
            [default]Folder/myTag
        collisionPolicy (str): The action to take when a tag or folder
            with the same path and name is encountered. Possible values
            include: "a" Abort and throw an exception, "o" Overwrite and
            replace existing Tag's configuration, "i" Ignore that item
            in the list. Defaults to Overwrite. Optional.

    Returns:
        list[QualityCode]: A List of QualityCode objects, one for each
            tag in the list, that is representative of the result of the
            operation.
    """
    print(tags, destination, collisionPolicy)
    return [QualityCode()]


def queryAnnotations(paths, startTime=None, endTime=None, types=None):
    """Queries user stored annotations from the tag history system for a
    set of paths, for a given time range.

    Args:
        paths (list[str]): A list of tag paths to query. The paths are
            equivalent to what would be used ofr a tag history query,
            and should specify the source provider as well. For example,
            "[HistoryProvider/Gateway:Provider]Path/To/Tag".
        startTime (Date): The start of the time range. If not defined,
            defaults to 12 hours ago. Optional.
        endTime (Date): The end of time range. If not defined, defaults
            to "now". Optional.
        types (list[object]): A list of string "types" to filter on.
            Types are defined by the annotations and various subsystems,
            and may vary with different providers. Possible annotation
            types are listed on the system.tag.storeAnnotations page.
             Optional.

    Returns:
        list[object]: A list of Annotation objects that match the query
            criteria.
    """
    print(paths, startTime, endTime, types)


def queryTagCalculations(
    paths,
    calculations,
    startDate=None,
    endDate=None,
    rangeHours=None,
    rangeMinutes=None,
    aliases=None,
    includeBoundingValues=True,
    validatesSCExec=True,
    noInterpolation=False,
    ignoreBadQuality=False,
):
    """Queries various calculations (aggregations) for a set of tags
    over a specified range.

    Returns a dataset with a row per tag, and a column per calculation.

    This is useful when you wish to aggregate tag history collected over
    a period of time into a single value per aggregate. If you want
    multiple values aggregated to a single time slice (i.e., hourly
    aggregates for the same tag over an 8 hour period) consider using
    system.tag.queryTagHistory

    Args:
        paths (list[str]): An array of tag paths (strings) to query
            calculations for. The resulting dataset will have a row for
            each tag, and a column for each calculation.
        calculations (list[str]): An array of calculations (aggregation
            functions) to execute for each tag. Valid values are:
            "Average" (time-weighted), "MinMax", "LastValue",
            "SimpleAverage", "Sum", "Minimum", "Maximum", "DurationOn",
            "DurationOff", "CountOn", "CountOff", "Count", "Range",
            "Variance", "StdDev", "PctGood", and "PctBad".
        startDate (Date): The starting point for the calculation
            window. If omitted, and range is not used, 8 hours before
            the current time is used. Optional.
        endDate (Date): The end of the calculation window. If
            omitted, and range is not used, uses the current time.
            Optional.
        rangeHours (int): Allows you to specify the query range in
            hours, instead of using start and end date. Can be positive
            or negative, and can be used in conjunction with startDate
            or endDate. Optional.
        rangeMinutes (int): Same as rangeHours, but in minutes.
            Optional.
        aliases (list[str]): Aliases that will be used to override the
            tag path names in the result dataset. Must be 1-to-1 with
            the tag paths. If not specified, the tag paths themselves
            will be used. Optional.
        includeBoundingValues (bool): A boolean flag indicating that the
            system should attempt to load values before and after the
            query bounds for the purpose of interpolation. The effect
            depends on the aggregates used. The default is "True".
            Optional.
        validatesSCExec (bool): A boolean flag indicating whether or not
            data should be validated against the scan class execution
            records. If False, calculations may include data that is
            assumed to be good, even though the system may not have been
            running. Default is "True". Optional.
        noInterpolation (bool): A boolean flag indicating that the
            system should not attempt to interpolate values in
            situations where it normally would, such as for analog tags.
            Default is "False". Optional.
        ignoreBadQuality (bool): A boolean flag indicating that bad
            quality values should not be used in the query process. If
            set, any value with a "bad" quality will be completely
            ignored in calculations. Default is "False". Optional.

    Returns:
        BasicDataset: A dataset representing the calculations over the
            specified range.
    """
    endDate = system.date.now() if endDate is None else endDate
    startDate = (
        system.date.addHours(endDate, -8) if startDate is None else startDate
    )
    print(
        paths,
        calculations,
        startDate,
        endDate,
        rangeHours,
        rangeMinutes,
        aliases,
        includeBoundingValues,
        validatesSCExec,
        noInterpolation,
        ignoreBadQuality,
    )
    return BasicDataset()


def queryTagDensity(paths, startDate, endDate):
    """Queries the Tag history system for information about the density
    of data.

    In other words, how much data is available for a given time span.

    This function is called with a list of Tag paths, and a start and
    end date. The result set is a two column dataset specifying the
    timestamp, and a relative weight. Each row is valid from the given
    time until the next row. Tags are assigned a 1 or a 0 if they are
    present or not. All values are then multiplied together to get a
    decimal based percentage for the density. Thus, for four Tag paths
    passed in, if three Tags were present during the span, the result
    would be 0.75.

    Args:
        paths (list[str]): An array of Tag paths (strings) to query.
        startDate (Date): The start of the range to query.
        endDate (Date): The end of the range to query.

    Returns:
        BasicDataset: A 2-column dataset consisting of a timestamp and a
            weight. Each row is valid until the next row.
    """
    print(paths, startDate, endDate)
    return BasicDataset()


def queryTagHistory(
    paths,
    startDate=None,
    endDate=None,
    returnSize=-1,
    aggregationMode="Average",
    returnFormat="Wide",
    columnNames=None,
    intervalHours=None,
    intervalMinutes=None,
    rangeHours=None,
    rangeMinutes=None,
    aggregationModes=None,
    includeBoundingValues=None,
    validateSCExec=None,
    noInterpolation=None,
    ignoreBadQuality=None,
    timeout=None,
    intervalSeconds=None,
    rangeSeconds=None,
):
    """Issues a query to the Tag Historian.

    Querying tag history involves specifying the tags and the date
    range, as well as a few optional parameters. The Tag historian will
    find the relevant history and then interpolate and aggregate it
    together into a coherent, tabular result set.

    Args:
        paths (list[str]): An array of tag paths (strings) to query.
            Each tag path specified will be a column in the result
            dataset.
        startDate (Date): The earliest value to retrieve. If
            omitted, 8 hours before current time is used. Optional.
        endDate (Date): The latest value to retrieve. If omitted,
            current time is used. Optional.
        returnSize (int): The number of samples to return. -1 will
            return values as they changed, and 0 will return the
            "natural" number of values based on the logging rates of the
            scan class(es) involved. -1 is the default. Optional.
        aggregationMode (str): The mode to use when aggregating multiple
            samples into one time slice. Valid values are: "Average"
            (time-weighted), "MinMax", "LastValue", "SimpleAverage",
            "Sum", "Minimum", "Maximum", "DurationOn", "DurationOff",
            "CountOn", "CountOff", "Count", "Range", "Variance",
            "StdDev", "PctGood", and "PctBad". Optional.
        returnFormat (str): Use "Wide" to have a column per tag queried,
            or "Tall" to have a fixed-column format. Default is "Wide".
            Optional.
        columnNames (list[str]): Aliases that will be used to override
            the column names in the result dataset. Must be 1-to-1 with
            the tag paths. If not specified, the tag paths themselves
            will be used as column titles. Optional.
        intervalHours (int): Allows you to specify the window interval
            in terms of hours, as opposed to using a specific return
            size. Optional.
        intervalMinutes (int): Same as intervalHours, but in minutes.
            Can be used on its own, or in conjunction with
            intervalHours. Optional.
        rangeHours (int): Allows you to specify the query range in
            hours, instead of using start and end date. Can be positive
            or negative, and can be used in conjunction with startDate
            or endDate. Optional.
        rangeMinutes (int): Same as rangeHours, but in minutes.
            Optional.
        aggregationModes (list[str]): A one-to-one list with paths
            specifying an aggregation mode per column. Optional.
        includeBoundingValues (bool): A boolean flag indicating that the
            system should attempt to include values for the query bound
            times if possible. The default for this property depends on
            the query mode, so unless a specific behavior is desired, it
            is best to not include this parameter. Optional.
        validateSCExec (bool): A boolean flag indicating whether or not
            data should be validated against the scan class execution
            records. If False, data will appear flat (but good quality)
            for periods of time in which the system wasn't running. If
            True, the same data would be bad quality during downtime
            periods. Optional.
        noInterpolation (bool): A boolean flag indicating that the
            system should not attempt to interpolate values in
            situations where it normally would. This will also prevent
            the return of rows that are purely interpolated. Optional.
        ignoreBadQuality (bool): A boolean flag indicating that bad
            quality values should not be used in the query process. If
            set, any value with a "bad" quality will be completely
            ignored in calculations and in the result set. Optional.
        timeout (int): Timeout in milliseconds for Client Scope. This
            property is ignored in the Gateway Scope. Optional.
        intervalSeconds (int): Same as intervalHours and interval
            Minutes, but in seconds. Can be used on its own, or in
            conjunction with intervalHours and intervalMinutes.
            Optional.
        rangeSeconds (int): Allows you to specify the query range in
            seconds, instead of using start and end date. Can be
            positive or negative, and can be used in conjunction with
            startDate or endDate. Optional.

    Returns:
        BasicDataset: A dataset representing the historian values for
            the specified tag paths. The first column will be the
            timestamp, and each column after that represents a tag.
    """
    endDate = system.date.now() if endDate is None else endDate
    startDate = (
        system.date.addHours(endDate, -8) if startDate is None else startDate
    )
    print(
        paths,
        startDate,
        endDate,
        returnSize,
        aggregationMode,
        returnFormat,
        columnNames,
        intervalHours,
        intervalMinutes,
        rangeHours,
        rangeMinutes,
        aggregationModes,
        includeBoundingValues,
        validateSCExec,
        noInterpolation,
        ignoreBadQuality,
        timeout,
        intervalSeconds,
        rangeSeconds,
    )
    return BasicDataset()


def read(tagPath):
    """Reads the value of the tag at the given tag path.

    Returns a qualified value object. You can read the value, quality,
    and timestamp from this object. If the tag path does not specify a
    tag property, then the Value property is assumed.

    You can also read the value of tag attributes by appending the
    attribute to the tagPath parameter. See the Tag Attributes page for
    a list of available attributes.

    Args:
        tagPath (str): Reads from the given tag path. If no property is
            specified in the path, the Value property is assumed.

    Returns:
        BasicQualifiedValue: A qualified value. This object has three
            sub-members: value, quality, and timestamp.
    """
    warnings.warn(
        "read is deprecated, use readAsync or readBlocking instead.",
        DeprecationWarning,
    )
    print(tagPath)
    return BasicQualifiedValue()


def readAll(tagPaths):
    """Reads the values of each tag in the tag path list.

    Returns a sequence of qualified value objects. You can read the
    value, quality, and timestamp from each object in the return
    sequence. Reading in bulk like this is more efficient than calling
    read() many times.

    Args:
        tagPaths (list[str]): A sequence of tag paths to read from.

    Returns:
        list[QualifiedValue]: A sequence of qualified values
            corresponding to each tag path given. Each qualified value
            will have three sub-members: value, quality, and timestamp.
    """
    warnings.warn(
        "readAll is deprecated, use readAsync or readBlocking instead.",
        DeprecationWarning,
    )
    print(tagPaths)
    return [BasicQualifiedValue() for _ in range(len(tagPaths))]


def readAsync(tagPaths, callback):
    """Asynchronously reads the value of the Tags at the given paths.

    You must provide a python callback function that can process the
    read results.

    Args:
        tagPaths (list[str]): A List of Tag paths to read from. If no
            property is specified in the path, the Value property is
            assumed.
        callback (object): A Python callback function to process the
            read results. The function definition must provide a single
            argument, which will hold a List of qualified values when
            the callback function is invoked. The qualified values will
            have three sub members: value, quality, and timestamp.
    """
    print(tagPaths, callback)


def readBlocking(tagPaths, timeout=45000):
    """Reads the value of the Tags at the given paths.

    Will block until the read operation is complete or times out.

    Args:
        tagPaths (list[str]): A List of Tag paths to read from. If no
            property is specified in a path, the Value property is
            assumed.
        timeout (int): How long to wait in milliseconds before the read
            operation times out. This parameter is optional, and
            defaults to 45000 milliseconds if not specified. Optional.

    Returns:
        list[BasicQualifiedValue]: A list of QualifiedValue objects
            corresponding to the Tag paths. Each qualified value will
            have three sub members: value, quality, and timestamp.
    """
    print(tagPaths, timeout)
    return [BasicQualifiedValue() for _ in tagPaths]


def rename(tag, newName, collisionPollicy="a"):
    """Renames a single tag or a folder.

    Args:
        tag (str): The tag path to rename.
        newName (str): The new name.
        collisionPollicy (str): The action to take when a tag or folder
            with the same path and names is encountered. Possible values
            include "a" (Abort, throws an exception), "o" (Overwrite,
            completely replaces a tag's configuration), and "i"
            (Ignore). Defaults to Abort if not specified.

    Returns:
        object: A quality code that contains the result of the rename
            operation.
    """
    print(tag, newName, collisionPollicy)


def requestGroupExecution(provider, tagGroup):
    """Sends a request to the specified Tag Group to execute now.

    Args:
        provider (str): Name of the Tag Provider that the Tag Group is
            in.
        tagGroup (str): The name of the Tag Group to execute.
    """
    print(provider, tagGroup)


def setOverlaysEnabled(enabled):
    """Enables or disables the component quality overlay system.

    Args:
        enabled (bool): True (1) to turn on tag overlays, False (0) to
            turn them off.
    """
    print(enabled)


def storeAnnotations(
    paths,
    startTimes=None,
    endTimes=None,
    types=None,
    data=None,
    storageIds=None,
    deleted=None,
):
    """Stores annotations into the tag history system.

    Annotations are stored by the underlying historian implementations,
    so different providers may store in different ways, and some
    providers may not support annotation storage. All parameters are
    1-to-1, so all provided lists should be of the same length. If a
    particular annotation doesn't need a parameter, that element can be
    None in the list.

    Args:
        paths (list[str]): A list of tag paths to store for. The paths
            are equivalent to what would be used for a tag history
            query, and should specify the source provider as well. For
            example, "[HistoryProvider/Gateway:Provider]Path/To/Tag".
            This parameter is required, even if storage ids are
            included, because it is used to identify the underlying
            storage provider.
        startTimes (list[Date]): The start times of the events. If
            omitted, defaults to the current time. Optional.
        endTimes (list[Date]): The end times of the event, if
            applicable. If omitted, does not store an end time for the
            annotation. Optional.
        types (list[str]): The type id for the annotation. If not
            defined, "marker" will be used. See the Annotation Types for
            more details. Optional.
        data (list[str]): Data for the annotation, such as text
            describing the meaning of the annotation. Optional.
        storageIds (list[int]): If defined, the function will instead
            update the existing annotation instead of adding new ones,
            overriding existing values for the annotation with those
            provided by this function (if the corresponding delete
            parameter is True). Storage id is available on the
            Annotation object, and is returned as the result value from
            the storeAnnotations call. Optional.
        deleted (list[bool]): A list of booleans indicating that the
            individual annotation should be deleted. Requires storage id
            to be set as well. Optional.

    Returns:
        list[QualifiedValue]: A list of qualified values. The quality
            code will indicate success or failure, and if successful,
            the storage id of the annotation will be returned in the
            value. Since annotations are stored by individual providers,
            there is no guarantee as to the type or format of the
            storage id. However, it can be held and re-used in order to
            update or delete.
    """
    print(paths, startTimes, endTimes, types, data, storageIds, deleted)


def storeTagHistory(
    historyprovider,
    tagprovider,
    paths,
    values,
    qualities=None,
    timestamps=None,
):
    """Inserts data into the tag history system, allowing Tag history to
    be recorded via scripting.

    The Tag paths are associated with a historical and realtime
    provider, but they do not necessarily need to exist in the realtime
    provider. This means records from non-existent (virtual) Tags can be
    stored in the Tag History system. Because of this, it is imperative
    that Tag paths passed to the function are typed precisely, otherwise
    the history will be stored at an incorrect path.

    Note that the Tag History system does cache tag data. Thus, if this
    function is called, the tag path and tag id are cached until the
    history provider or gateway are restarted. This means manually
    removing the tag from the sqlth_te table, and then calling this
    function again with the same path will not re-populate the tag
    execution table (especially so when working purely with virtual tag
    paths). Instead, the cache must first be cleared, and then a new
    entry will be added the next time this function is called.

    Args:
        historyprovider (str): The historical provider to store to.
        tagprovider (str): The name of the realtime tag provider to
            associate these tags with. The tag provider does not need to
            exist, and the tag paths do not need to exist in it.
        paths (list[str]): A list of paths to store. The values,
            qualities, and timestamps are one-to-one with the paths. A
            single path may be present multiple times in order to store
            multiple values.
        values (list[object]): A list of values to store.
        qualities (list[int]): A list of integer quality codes
            corresponding to the values. Quality codes can be found on
            the Tag Quality and Overlays page. If omitted, GOOD quality
            will be used. Optional.
        timestamps (list[Date]): A list of Date timestamps corresponding
            to the values. If omitted, the current time will be used. A
            java.util.date object may be passed, so the system.date
            functions can be used to return a timestamp. Optional.
    """
    print(historyprovider, tagprovider, paths, values, qualities, timestamps)


def write(tagPath, value, suppressErrors=False):
    """Writes a value to a tag.

    Note that this function writes asynchronously. This means that the
    function does not wait for the write to occur before returning - the
    write occurs sometime later on a different thread.

    Args:
        tagPath (str): The path of the tag to write to.
        value (object): The value to write.
        suppressErrors (bool): A flag indicating whether or not to
            suppress errors. Optional.

    Returns:
        int: 0 if the write failed immediately, 1 if it succeeded
            immediately, and 2 if it is pending.
    """
    warnings.warn(
        "write is deprecated, use writeAsync or writeBlocking instead.",
        DeprecationWarning,
    )
    print(tagPath, value, suppressErrors)
    return 1


def writeAll(tagPaths, values):
    """Performs an asynchronous bulk write.

    Takes two sequences that must have the same number of entries. The
    first is the list of tag paths to write to, and the second is a list
    of values to write. This function is dramatically more efficient
    than calling write multiple times.

    Args:
        tagPaths (list[str]): The paths of the tags to write to.
        values (list[object]): The values to write.

    Returns:
        list[int]: Array of ints with an element for each tag written
            to: 0 if the write failed immediately, 1 if it succeeded
            immediately, and 2 if it is pending.
    """
    warnings.warn(
        "writeAll is deprecated, use writeAsync or writeBlocking instead.",
        DeprecationWarning,
    )
    print(tagPaths, values)
    return [1] * len(tagPaths)


def writeAsync(tagPaths, values, callback=None):
    """Asynchronously writes values to Tags a the given paths.

    You must provide a Python callback function that can process the
    write results.

    Args:
        tagPaths (list[str]): A List of Tag paths to write to. If no
            property is specified in a Tag path, the Value property is
            assumed.
        values (list[object]): The values to write to the specified
            paths.
        callback (object): A Python callback function to process the
            write results. The function definition must provide a single
            single argument, which will hold a List of quality codes
            when the callback function is invoked. The quality codes
            will hold the result of the write operation for that Tag.
            Optional.
    """
    print(tagPaths, values, callback)


def writeBlocking(tagPaths, values, timeout=45000):
    """Writes values to Tags at the given paths.

    This function will block until the write operation is complete or
    times out.

    Args:
        tagPaths (list[str]): A List of Tag paths to write to. If no
            property is specified in a Tag path, the Value property is
            assumed.
        values (list[object]): The values to write to the specified
            paths.
        timeout (int): How long to wait in milliseconds before the write
            operation times out. This parameter is optional, and
            defaults to 45000 milliseconds if not specified.

    Returns:
        list[QualityCode]: A List of QualityCode objects, one for each
            Tag path. Each quality code holds the result of the write
            operation for that Tag.
    """
    print(tagPaths, values, timeout)
    return [QualityCode()]



from __future__ import print_function

__all__ = ["BasicUser", "ContactInfo", "PyUser", "User"]

from java.lang import Object


class ContactInfo(Object):
    def __init__(self, contactType=None, value=None):
        self.contactType = contactType
        self.value = value

    def getContactType(self):
        pass

    def getValue(self):
        pass

    def setContactType(self, contactType):
        pass

    def setValue(self, value):
        pass


class User(object):
    _Badge = "badge"
    _DEFAULT_SCHEDULE_NAME = "Always"
    _FirstName = "John"
    _Language = "en"
    _LastName = "Doe"
    _Notes = "These are some notes."
    _Password = "password"
    _Schedule = "Always"
    _Username = "johdoe"
    _USERNAME_PATTERN = r"[\p{Alnum}][ @\w.\s\-]{1, 49}"

    @property
    def Badge(self):
        return self._Badge

    @property
    def DEFAULT_SCHEDULE_NAME(self):
        return self._DEFAULT_SCHEDULE_NAME

    @property
    def FirstName(self):
        return self._FirstName

    @property
    def Language(self):
        return self._Language

    @property
    def LastName(self):
        return self._LastName

    @property
    def Notes(self):
        return self._Notes

    @property
    def Password(self):
        return self._Password

    @property
    def Schedule(self):
        return self._Schedule

    @property
    def Username(self):
        return self._Username

    @property
    def USERNAME_PATTERN(self):
        return self._USERNAME_PATTERN

    def getContactInfo(self):
        raise NotImplementedError

    def getId(self):
        raise NotImplementedError

    def getPath(self):
        raise NotImplementedError

    def getProfileName(self):
        raise NotImplementedError

    def getRoles(self):
        raise NotImplementedError

    def getScheduleAdjustments(self):
        raise NotImplementedError


class BasicUser(User):
    def __init__(self, profileName, id, roles, contactInfo=None):
        self.profileName = profileName
        self.id = id
        self.roles = roles
        self.contactInfo = contactInfo

    def getContactInfo(self):
        pass

    def getId(self):
        pass

    def getPath(self):
        pass

    def getProfileName(self):
        pass

    def getRoles(self):
        pass

    def getScheduleAdjustments(self):
        pass


class PyUser(User):
    """A User implementation that delegates to another user object, but
    has some methods that are more scripting friendly.
    """

    _user = None

    def __init__(self, user=None):
        self._user = user

    def addContactInfo(self, *args):
        """Convenience method for scripting to add a new contactInfo
        easily.

        Usage:
            1. addContactInfo(contactType: str, value: str)
            2. addContactInfo(dict: contactDictionary)

        Args:
            args: Variable length argument list.
        """
        print(self, args)

    def addRole(self, role):
        """Convenience method for scripting to add a new role easily.

        Only works if user type supports roles.

        Args:
            role (str): A new role to add. If empty or null, no
                effect.
        """
        print(self, role)

    def addRoles(self, roles):
        """Adds the provided roles to this user.

        Args:
            roles (list[str]): A list of roles.
        """
        print(self, roles)

    def addScheduleAdjustment(self, start, end, available=True, note=None):
        """Convenience method for scripting to add a new schedule
        adjustment easily.

        Args:
            start (Date): Date to start the schedule adjustment. Not
                null.
            end (Date): Date to end start the schedule adjustment.
                Not null.
            available (bool): True if the employee is available during
                this period. Optional.
            note (str): May be null or empty. Optional.
        """
        print(self, start, end, available, note)

    def addScheduleAdjustments(self, scheduleAdjustments):
        """Add Schedule Adjustments.

        Args:
            scheduleAdjustments (ScheduleAdjustment): ScheduleAdjustment
                object.
        """
        print(self, scheduleAdjustments)

    def contains(self, prop):
        """Returns if this users contains a given property."""
        pass

    def get(self, propertyName):
        """Returns a the value of the requested item.

        Args:
            propertyName (str): The user property to retrieve.

        Returns:
            str: The value of the requested property.
        """
        print(self)
        return propertyName

    def getContactInfo(self):
        """Returns a sequence of ContactInfo objects.

        Each of these objects will have a contactType and value property
        representing the contact information, both strings.

        Returns:
            list[ContactInfo]: A sequence of ContactInfo objects.
        """
        ci_email = ContactInfo("email", "johdoe@mycompany.com")
        ci_phone = ContactInfo("phone", "+1 5551324567")
        ci_sms = ContactInfo("sms", "+1 5557654321")
        return [ci_email, ci_phone, ci_sms]

    def getCount(self):
        """Get count."""
        print(self)
        return 1

    def getId(self):
        """An opaque identifier that can be used to identify this
        user.
        """
        pass

    def getOrDefault(self, prop):
        """Returns a default value if the requested item is not
        present.

        Args:
            prop (Property): The user property to retrieve.

        Returns:
            object: The value of the requested property.
        """
        print(self, prop)

    def getOrElse(self, property, value):
        """Get the value for a given Property, or else fall back to
        value if it's not present.
        """
        pass

    def getPath(self):
        """Generate an path that unambiguously references this user."""
        pass

    def getProfileName(self):
        """The name of the user management profile this user was
        retrieved from.
        """
        pass

    def getProperties(self):
        """Returns all properties for this user."""
        pass

    def getRoles(self):
        """Returns all of the roles this user is a has."""
        return ["Administrator", "Developer"]

    def getScheduleAdjustments(self):
        """Returns all of this user's upcoming schedule adjustments."""
        pass

    def getValues(self):
        """Returns the opaque PropertyValue objects."""
        pass

    def isExtended(self, prop):
        """Returns whether this property set contains a value for the
        prop, and the prop was actually inherited.
        """
        pass

    def isInherited(self, prop):
        """Indicates whether the property was inherited from a parent
        type.
        """
        pass

    def iterator(self):
        """Property iterator."""
        pass

    def merge(self, other, localOnly):
        """Merges the values from other collection into this one."""
        pass

    def remove(self, prop):
        """Remove a property for this user."""
        pass

    def removeContactInfo(self, contactType, value):
        """Remove contact information for this user."""
        pass

    def removeRole(self, role):
        """Remove a role for this user."""
        pass

    def removeScheduleAdjustment(self, start, end, available=True, note=None):
        """Remove schedule adjustment for this user."""
        pass

    def set(self, *args):
        """Set a property for this user."""
        pass

    def setContactInfo(self, contactInfo):
        """Set contact information for this user."""
        pass

    def setRoles(self, roles):
        """Set roles for this user."""
        pass

    def setScheduleAdjustments(self, scheduleAdjustments):
        """Set schedule adjustments for this user."""
        pass



"""Provides classes for creating and modifying images.

Images are processed using a streaming framework that involves an image
producer, optional image filters, and an image consumer. This framework
makes it possible to progressively render an image while it is being
fetched and generated. Moreover, the framework allows an application to
discard the storage used by an image and to regenerate it at any time.
This package provides a number of image producers, consumers, and
filters that you can configure for your image processing needs.
"""

from __future__ import print_function

__all__ = ["BufferedImage"]

from java.awt import Image


class BufferedImage(Image):
    """The BufferedImage subclass describes an Image with an accessible
    buffer of image data.

    A BufferedImage is comprised of a ColorModel and a Raster of image
    data. The number and types of bands in the SampleModel of the Raster
    must match the number and types required by the ColorModel to
    represent its color and alpha components. All BufferedImage objects
    have an upper left corner coordinate of (0, 0). Any Raster used to
    construct a BufferedImage must therefore have minX=0 and minY=0.

    This class relies on the data fetching and setting methods of
    Raster, and on the color characterization methods of ColorModel.
    """

    def __init__(self, *args):
        """Constructs a BufferedImage.

        Args:
            *args: Variable length argument list.
        """
        print(args)



"""Provides classes that are fundamental to the design of the Java
programming language.
"""

from __future__ import print_function

__all__ = [
    "Exception",
    "IllegalArgumentException",
    "Object",
    "RuntimeException",
    "Thread",
    "Throwable",
]

import copy
import time

try:
    import __builtin__ as builtins
except ImportError:
    import builtins


class Object(object):
    """Class Object is the root of the class hierarchy. Every class has
    Object as a superclass. All objects, including arrays, implement the
    methods of this class.
    """

    def clone(self):
        """Creates and returns a copy of this object.

        Returns:
            object: A copy of this object.
        """
        return copy.deepcopy(self)

    def equals(self, obj):
        """Indicates whether some other object is "equal to" this one.

        The equals method implements an equivalence relation on non-null
        object references:

            - It is reflexive: for any non-null reference value x,
            x.equals(x) should return True.
            - It is symmetric: for any non-null reference values x and
            y, x.equals(y) should return True if and only if y.equals(x)
            returns True.
            - It is transitive: for any non-null reference values x, y,
            and z, if x.equals(y) returns True and y.equals(z) returns
            True, then x.equals(z) should return True.
            - It is consistent: for any non-null reference values x and
            y, multiple invocations of x.equals(y) consistently return
            True or consistently return False, provided no information
            used in equals comparisons on the objects is modified.
            - For any non-null reference value x, x.equals(null) should
            return False.

        The equals method for class Object implements the most
        discriminating possible equivalence relation on objects; that
        is, for any non-null reference values x and y, this method
        returns True if and only if x and y refer to the same object
        (x == y has the value True).

        Note that it is generally necessary to override the hashCode
        method whenever this method is overridden, so as to maintain the
        general contract for the hashCode method, which states that
        equal objects must have equal hash codes.

        Args:
            obj (object): The reference object with which to compare.

        Returns:
            bool: True if this object is the same as the obj argument;
                False otherwise.
        """
        print(self, obj)
        return True

    def getClass(self):
        """Returns the runtime class of this Object."""
        pass

    def hashCode(self):
        """Returns a hash code value for the object."""
        pass

    def notify(self):
        """Wakes up a single thread that is waiting on this object's
        monitor.
        """
        pass

    def notifyAll(self):
        """Wakes up all threads that are waiting on this object's
        monitor.
        """
        pass

    def toString(self):
        """Returns a string representation of the object. In general,
        the toString method returns a string that "textually represents"
        this object. The result should be a concise but informative
        representation that is easy for a person to read. It is
        recommended that all subclasses override this method.

        Returns:
             str: A string representation of the object.
        """
        return repr(self)

    def wait(self, timeoutMillis=None, nanos=None):
        """Causes the current thread to wait until it is awakened,
        typically by being notified or interrupted, or until a certain
        amount of real time has elapsed.

        Args:
            timeoutMillis (long): The maximum time to wait, in
                milliseconds.
            nanos (int): Additional time, in nanoseconds, in the range
                range 0-999999 inclusive.
        """
        pass


class Throwable(Object, builtins.Exception):
    """The Throwable class is the superclass of all errors and
    exceptions in the Java language.
    """

    def __init__(self, message=None, cause=None):
        """Constructs a new throwable.

        Args:
            message (str): The detail message (which is saved for later
                retrieval by the getMessage() method).
            cause (Throwable): The cause (which is saved for later
                retrieval by the getCause() method). (A null value is
                permitted, and indicates that the cause is nonexistent
                or unknown.).
        """
        self._cause = cause
        self.message = message
        builtins.Exception.__init__(self, message)

    @property
    def cause(self):
        """Cause property."""
        return self._cause

    def addSuppressed(self, exception):
        """Appends the specified exception to the exceptions that were
        suppressed in order to deliver this exception.

        Args:
            exception (Throwable): The exception to be added to the list
                of suppressed exceptions

        Raises:
            IllegalArgumentException: If exception is this throwable; a
                throwable cannot suppress itself.
            NullPointerException: If exception is null.
        """
        pass

    def fillInStackTrace(self):
        """Fills in the execution stack trace."""
        pass

    def getCause(self):
        """Returns the cause of this throwable or null if the cause is
        nonexistent or unknown.
        """
        return self.cause

    def getLocalizedMessage(self):
        """Creates a localized description of this throwable."""
        return self.message

    def getMessage(self):
        """Returns the detail message string of this throwable."""
        return self.message

    def getStackTrace(self):
        """Provides programmatic access to the stack trace information
        printed by printStackTrace().
        """
        pass

    def getSuppressed(self):
        """Returns an array containing all of the exceptions that were
        suppressed, typically by the try-with-resources statement, in
        order to deliver this exception.
        """
        pass

    def initCause(self, cause=None):
        """Initializes the cause of this throwable to the specified
        value.

        Args:
            cause (Throwable): The cause (which is saved for later
                retrieval by the getCause() method). (A null value is
                permitted, and indicates that the cause is nonexistent
                or unknown.)

        Returns:
            Throwable: A reference to this Throwable instance.
        """
        pass

    def printStackTrace(self, *args):
        """Prints this throwable and its backtrace."""
        pass

    def setStackTrace(self, stackTrace):
        """Sets the stack trace elements that will be returned by
        getStackTrace() and printed by printStackTrace() and related
        methods.
        """
        pass

    def toString(self):
        """Returns a short description of this throwable."""
        return "A short description of this throwable."


class Exception(Throwable):
    """The class Exception and its subclasses are a form of Throwable
    that indicates conditions that a reasonable application might want
    to catch.

    The class Exception and any subclasses that are not also subclasses
    of RuntimeException are checked exceptions. Checked exceptions need
    to be declared in a method or constructor's throws clause if they
    can be thrown by the execution of the method or constructor and
    propagate outside the method or constructor boundary.
    """

    def __init__(self, message=None, cause=None):
        """Constructs an Exception."""
        super(Exception, self).__init__(message, cause)


class RuntimeException(Exception):
    """RuntimeException is the superclass of those exceptions that can
    be thrown during the normal operation of the Java Virtual Machine.

    RuntimeException and its subclasses are unchecked exceptions.
    Unchecked exceptions do not need to be declared in a method or
    constructor's throws clause if they can be thrown by the execution
    of the method or constructor and propagate outside the method or
    constructor boundary.
    """

    def __init__(self, message=None, cause=None):
        """Constructs a RuntimeException."""
        super(RuntimeException, self).__init__(message, cause)


class IllegalArgumentException(RuntimeException):
    """Thrown to indicate that a method has been passed an illegal or
    inappropriate argument.
    """

    def __init__(self, message=None, cause=None):
        """Constructs an IllegalArgumentException."""
        super(IllegalArgumentException, self).__init__(message, cause)


class NullPointerException(RuntimeException):
    """Thrown when an application attempts to use null in a case where
    an object is required.
    """

    def __init__(self, message=None, cause=None):
        """Constructs a NullPointerException."""
        super(NullPointerException, self).__init__(message, cause)


class Thread(Object):
    """A thread is a thread of execution in a program. The Java Virtual
    Machine allows an application to have multiple threads of execution
    running concurrently.

    Every thread has a name for identification purposes. More than one
    thread may have the same name. If a name is not specified when a
    thread is created, a new name is generated for it.

    Unless otherwise noted, passing a null argument to a constructor or
    method in this class will cause a NullPointerException to be thrown.
    """

    @staticmethod
    def sleep(millis):
        """Causes the currently executing thread to temporarily cease
        execution for the specified number of milliseconds, subject to
        the precision and accuracy of system timers and schedulers. The
        thread does not lose ownership of any monitors.

        Args:
            millis (long): The length of time to sleep in milliseconds.
        """
        time.sleep(millis // 1000)



"""EAM Functions.

The following functions give you access to view EAM information from the
Gateway.
"""

from __future__ import print_function

__all__ = ["getGroups", "queryAgentHistory", "queryAgentStatus", "runTask"]

import system.date
from com.inductiveautomation.ignition.common import BasicDataset
from com.inductiveautomation.ignition.common.messages import UIResponse
from java.util import Date, Locale


def getGroups():
    """Returns the names of the defined agent organizational groups in
    the Gateway.

    Returns:
        list[str]: A string list of group names.
    """
    return [""]


def queryAgentHistory(
    groupIds=None, agentIds=None, startDate=None, endDate=None, limit=100
):
    """Returns a list of the most recent agent events.

    Args:
        groupIds (list[str]): A list of groups to restrict the results
            to. If not specified, all groups will be included. Optional.
        agentIds (list[str]): A list of agent names to restrict the
            results to. If not specified, all agents will be allowed.
            Optional.
        startDate (Date): The starting time for history events. If
            null, defaults to 8 hours previous to now. Optional.
        endDate (Date): The ending time for the query range. If
            null, defaults to "now". Optional.
        limit (int): The limit of results to return. Defaults to 100. A
            value of 0 means "no limit". Optional.

    Returns:
        BasicDataset: A dataset with columns id, agent_name, agent_role,
            event_time, event_category, event_type, event_source,
            event_level, event_level_int, and message, where each row is
            a new agent event.
    """
    endDate = system.date.now() if endDate is None else endDate
    startDate = (
        system.date.addHours(endDate, -8) if startDate is None else startDate
    )
    print(groupIds, agentIds, startDate, endDate, limit)
    return BasicDataset()


def queryAgentStatus(groupIds=None, agentIds=None, isConnected=True):
    """Returns the current state of the matching agents.

    Args:
        groupIds (list[str]): A list of groups to restrict the results
            to. If not specified, all groups will be included.
        agentIds (list[str]): A list of agent names to restrict the
            results to. If not specified, all agents will be allowed.
        isConnected (bool): If True, only returns agents that are
            currently connected. If False, only agents that are
            considered down will be returned, and if not specified, all
            agents will be returned.

    Returns:
        BasicDataset: A dataset with columns AgentName, NodeRole,
            AgentGroup, LastCommunication, IsConnected, IsRunning,
            RunningState, RunningStateInt, LicenseKey, and Version,
            where each row is a new agent.
    """
    print(groupIds, agentIds, isConnected)
    return BasicDataset()


def runTask(taskname):
    """Takes the name of a task as an argument as a string (must be
    configured on the Controller before hand), attempts to execute the
    task.

    To run in the client, the user needs a role-based permission. This
    permission is disabled by default.

    Args:
        taskname (str): Name of the task to run. If more than one task
            has this name, an error will be returned.

    Returns:
        UIResponse: A UIResponse with a list of infos, errors, and
            warnings.
    """
    print(taskname)
    return UIResponse(Locale.ENGLISH)



"""Dataset Functions.

The following functions give you access to view and interact with
datasets.
"""

from __future__ import print_function

__all__ = [
    "addColumn",
    "addRow",
    "addRows",
    "appendDataset",
    "clearDataset",
    "dataSetToHTML",
    "deleteRow",
    "deleteRows",
    "exportCSV",
    "exportExcel",
    "exportHTML",
    "filterColumns",
    "formatDates",
    "fromCSV",
    "getColumnHeaders",
    "setValue",
    "sort",
    "toCSV",
    "toDataSet",
    "toExcel",
    "toPyDataSet",
    "updateRow",
]

import os

from com.inductiveautomation.ignition.common import BasicDataset, Dataset
from com.inductiveautomation.ignition.common.script.builtin import (
    DatasetUtilities,
)
from java.util import Locale


def addColumn(dataset, colIndex, col, colName, colType):
    """Takes a dataset and returns a new dataset with a new column added
    or inserted into it.

    Datasets are immutable, so it is important to realize that this
    function does not actually add a column to a dataset. You'll need to
    do something with the new dataset that this function creates to
    achieve something useful. If the columnIndex argument is omitted,
    the column will be appended to the end of the dataset.

    Args:
        dataset (Dataset): The starting dataset. Please be aware that
            this dataset will not actually be modified (datasets are
            immutable), but rather will be the starting point for
            creating a new dataset.
        colIndex (int): The index (starting at 0) at which to insert the
            new column. Will throw an IndexError if less than zero or
            greater than the length of the dataset. If omitted, the new
            column will be appended to the end. Optional.
        col (list[object]): A Python sequence representing the data for
            the new column. Its length must equal the number of rows in
            the dataset.
        colName (str): The name of the column.
        colType (type): The type of the of the column. The type can be
            the Python equivalent of String, Long, Double, Short,
            Integer, Float, Boolean, null, or java.util.Date if they
            exist.

    Returns:
        BasicDataset: A new dataset with the new column inserted or
            appended.
    """
    print(dataset, colIndex, col, colName, colType)
    return BasicDataset()


def addRow(dataset, rowIndex, row):
    """Takes a dataset and returns a new dataset with a new row added or
    inserted into it.

    Datasets are immutable, so it is important to realize that this
    function does not actually add a row to a dataset. You'll need to do
    something with the new dataset that this function creates to achieve
    something useful. If the rowIndex argument is omitted, the row will
    be appended to the end of the dataset.

    Args:
        dataset (Dataset): The starting dataset. Please be aware that
            this dataset will not actually be modified (datasets are
            immutable), but rather will be the starting point for
            creating a new dataset.
        rowIndex (int): The index (starting at 0) at which to insert the
            new row. Will throw an IndexError if less than zero or
            greater than the length of the dataset. If omitted, the new
            row will be appended to the end. Optional.
        row (list[object]): A Python sequence representing the data for
            the new row. Its length must equal the number of columns in
            the dataset.

    Returns:
        BasicDataset: A new dataset with the new row inserted or
            appended.
    """
    print(dataset, rowIndex, row)
    return BasicDataset()


def addRows(dataset, rowIndex, rows):
    """Takes a dataset and returns a new dataset with a new row added or
    inserted into it.

    Datasets are immutable, so it is important to realize that this
    function does not actually add a row to a dataset. You'll need to do
    something with the new dataset that this function creates to achieve
    something useful. If the rowIndex argument is omitted, the row will
    be appended to the end of the dataset.

    Args:
        dataset (Dataset): The starting dataset. Please be aware that
            this dataset will not actually be modified (datasets are
            immutable), but rather will be the starting point for
            creating a new dataset.
        rowIndex (int): The index (starting at 0) at which to insert the
            new row. Will throw an IndexError if less than zero or
            greater than the length of the dataset. If omitted, the new
            row will be appended to the end. Optional.
        rows (list[object]): A Python sequence representing the data for
            the new rows. The length of each sequence must equal the
            number of columns in the dataset.

    Returns:
        BasicDataset: A new dataset with the new row inserted or
            appended.
    """
    print(dataset, rowIndex, rows)
    return BasicDataset()


def appendDataset(dataset1, dataset2):
    """Takes two different datasets and returns a new dataset with the
    second dataset appended to the first.

    Will throw an error if the number and types of columns do not match.

    Args:
        dataset1 (Dataset): The dataset that will come first in the
            returned dataset.
        dataset2 (Dataset): The second dataset that will be appended to
            the end in the returned dataset.

    Returns:
        BasicDataset: A new dataset that is a combination of the
            original two datasets.
    """
    print(dataset1, dataset2)
    return BasicDataset()


def clearDataset(dataset):
    """Takes a dataset and returns a new dataset with all of the same
    column names, but all of the rows deleted.

    Args:
        dataset (Dataset): The starting dataset. If NULL, a NULL dataset
            will be returned.

    Returns:
        BasicDataset: A new dataset with no data.
    """
    print(dataset)
    return BasicDataset()


def dataSetToHTML(showHeaders, dataset, title):
    """Formats the contents of a dataset as an HTML page, returning the
    results as a string.

    Uses the <table> element to create a data table page.

    Args:
        showHeaders (bool): If True(1), the HTML table will include a
            header row.
        dataset (Dataset): The dataset to export.
        title (str): The title for the HTML page.

    Returns:
        str: The HTML page as a string.
    """
    print(showHeaders, dataset, title)
    return "<html><head>{}</head><body>data</body></html>".format(title)


def deleteRow(dataset, rowIndex):
    """Takes a dataset and returns a new dataset with a row removed.

    Datasets are immutable, so it is important to realize that this
    function does not actually remove the row from the argument dataset.
    You'll need to do something with the new dataset that this function
    creates to achieve something useful.

    Args:
        dataset (Dataset): The starting dataset. Please be aware that
            this dataset will not actually be modified (datasets are
            immutable), but rather will be the starting point for
            creating a new dataset.
        rowIndex (int): The index (starting at 0) of the row to delete.

    Returns:
        BasicDataset: A new dataset with the specified row removed.

    Raises:
        IndexError: If rowIndex is less than zero or greater than the
            row count of the dataset -1.
    """
    print(dataset, rowIndex)
    if rowIndex < 0:
        raise IndexError("Error")

    return BasicDataset()


def deleteRows(dataset, rowIndices):
    """Takes a dataset and returns a new dataset with one or more rows
    removed.

    Datasets are immutable, so it is important to realize that this
    function does not actually remove the rows from the argument
    dataset. You'll need to do something with the new dataset that this
    function creates to achieve something useful.

    Args:
        dataset (Dataset): The starting dataset. Please be aware that
            this dataset will not actually be modified (datasets are
            immutable), but rather will be the starting point for
            creating a new dataset.
        rowIndices (list[int]): The indices (starting at 0) of the rows
            to delete.

    Returns:
        BasicDataset: A new dataset with the specified row removed.

    Raises:
        IndexError: If any element is less than zero or greater than the
            number of rows in the dataset - 1.
    """
    print(dataset, rowIndices)
    if -1 in rowIndices:
        raise IndexError("Error")

    return BasicDataset()


def exportCSV(filename, showHeaders, dataset):
    """Exports the contents of a dataset as a CSV file, prompting the
    user to save the file to disk.

    Args:
        filename (str): A suggested filename to save as.
        showHeaders (bool): If True (1), the CSV file will include a
            header row.
        dataset (Dataset): The dataset to export.

    Returns:
        str: The path to the saved file, or None if the action was
            canceled by the user.
    """
    print(filename, showHeaders, dataset)
    return os.path.expanduser("~")


def exportExcel(filename, showHeaders, dataset, nullsEmpty=False):
    """Exports the contents of a dataset as an Excel spreadsheet,
    prompting the user to save the file to disk. Uses the same format as
    the toExcel function.

    Args:
        filename (str): A suggested filename to save as.
        showHeaders (bool): If True (1), the spreadsheet will include a
            header row.
        dataset (list[Dataset]): A sequence of datasets, one for each
            sheet in the resulting workbook.
        nullsEmpty (bool): If True (1), the spreadsheet will leave cells
            with NULL values empty, instead of allowing Excel to provide
            a default value like 0. Defaults to False. Optional.

    Returns:
        str: The path to the saved file, or None if the action was
            canceled by the user.
    """
    print(filename, showHeaders, dataset, nullsEmpty)
    return os.path.expanduser("~")


def exportHTML(filename, showHeaders, dataset, title):
    """Exports the contents of a dataset to an HTML page.

    Prompts the user to save the file to disk.

    Args:
        filename (str): A suggested filename to save as.
        showHeaders (bool): If True (1), the HTML table will include a
            header row.
        dataset (Dataset): The dataset to export.
        title (str): The title for the HTML page.

    Returns:
        str: The path to the saved file, or None if the action was
            canceled by the user.
    """
    print(filename, showHeaders, dataset, title)
    return os.path.expanduser("~")


def filterColumns(dataset, columns):
    """Takes a dataset and returns a view of the dataset containing only
    the columns found within the given list of columns.

    Args:
        dataset (Dataset): The starting dataset.
        columns (list[object]): A list of columns to keep in the
            returned dataset. The columns may be in integer index form
            (starting at 0), or the name of the columns as Strings.

    Returns:
        BasicDataset: A new dataset containing the filtered columns.
    """
    print(dataset, columns)
    return BasicDataset()


def formatDates(dataset, dateFormat, locale=Locale.ENGLISH):
    """Returns a new dataset with Date columns as strings formatted
    according to the dateFormat specified.

    Args:
        dataset (Dataset): The starting dataset to format.
        dateFormat (str): A valid Java DateFormat string such as
            "yyyy-MM-dd HH:mm:ss". See system.date.format for more
            information on the valid characters.
        locale (Locale): The Locale to use for formatting. If no Locale
            is specified, the default Locale will be used. Optional.

    Returns:
        BasicDataset: A new dataset, containing the formatted dates.
    """
    print(dataset, dateFormat, locale)
    return BasicDataset()


def fromCSV(csv):
    """Converts a dataset stored in a CSV formatted string to a dataset
    that can be immediately assignable to a dataset property in your
    project.

    Usually this is used in conjunction with
    system.file.readFileAsString when reading in a CSV file that was
    exported using system.dataset.toCSV. The CSV string must be
    formatted in a specific way.

    Args:
        csv (str): A string holding a CSV dataset.

    Returns:
        BasicDataset: A new dataset.
    """
    print(csv)
    return BasicDataset()


def getColumnHeaders(dataset):
    """Takes in a dataset and returns the headers as a python list.

    Args:
        dataset (Dataset): The input dataset.

    Returns:
        list[str]: A list of column header strings.
    """
    print(dataset)
    return []


def setValue(dataset, rowIndex, columnName, value):
    """Takes a dataset and returns a new dataset with a one value
    altered.

    Datasets are immutable, so it is important to realize that this
    function does not actually set a value in the argument dataset.
    You'll need to do something with the new dataset that this function
    creates to achieve something useful.

    Args:
        dataset (Dataset): The starting dataset. Will not be modified
            (datasets are immutable), but acts as the basis for the
            returned dataset.
        rowIndex (int): The index of the row to set the value at
            (starting at 0).
        columnName (str): The name of the column to set the value at.
            Case insensitive.
        value (object): The new value for the specified row/column.

    Returns:
         BasicDataset: A new dataset, with the new value set at the
            given location.
    """
    print(dataset, rowIndex, columnName, value)
    return BasicDataset()


def sort(dataset, keyColumn, ascending=True):
    """Sorts a dataset and returns the sorted dataset.

    This works on numeric, as well as alphanumeric columns. It will go
    character by character, going from 0-9, A-Z, a-z.

    Args:
        dataset (Dataset): The dataset to sort.
        keyColumn (object): The index or column name of the column to
            sort on.
        ascending (bool): True for ascending order, False for descending
            order. If omitted, ascending order will be used. Optional.

    Returns:
        BasicDataset: A new sorted dataset.
    """
    print(dataset, keyColumn, ascending)
    return BasicDataset()


def toCSV(dataset, showHeaders=True, forExport=False, localized=False):
    """Formats the contents of a dataset as CSV (comma separated
    values), returning the resulting CSV as a string.

    If the "forExport" flag is set, then the format will be appropriate
    for parsing using the system.dataset.fromCSV function.

    Args:
        dataset (Dataset): The dataset to export to CSV.
        showHeaders (bool): If set to True(1), a header row will be
            present in the CSV. Default is True(1). Optional.
        forExport (bool): If set to True(1), extra header information
            will be present in the CSV data which is necessary for the
            CSV to be compatible with the fromCSV method. Overrides
            showHeaders. Default is False(0). Optional.
        localized (bool): If set to True(1), the string representations
            of the values in the CSV data will be localized. Default is
            False(0). Optional.

    Returns:
        str: The CSV data as a string.
    """
    print(dataset, showHeaders, forExport, localized)
    return ""


def toDataSet(*args):
    """This function is used to convert PyDataSets to DataSets, and to
    create new datasets from raw Python lists.

    When creating a new dataset, headers should have unique names.

    1) system.dataset.toDataSet(dataset)
    2) system.dataset.toDataSet(headers, data)

    Args:
        args: A variable-length argument list.

    Returns:
        BasicDataset: The newly created dataset.
    """
    for arg in args:
        print(arg)
    return BasicDataset()


def toExcel(showHeaders, dataset, nullsEmpty=False, sheetNames=None):
    """Formats the contents of one or more datasets as an excel
    spreadsheet, returning the results as a string.

    Each dataset specified will be added as a worksheet in the Excel
    workbook. This function uses an xml-format for Excel spreadsheets,
    not the native Excel file format.

    Args:
        showHeaders (bool): If True, the spreadsheet will include a
            header row. If False, the header row will be omitted.
        dataset (list[Dataset]): A sequence of one or more datasets,
            one for each sheet in the resulting workbook.
        nullsEmpty (bool): If True (1), the spreadsheet will leave cells
            with NULL values empty, instead of allowing Excel to provide
            a default value like 0. Defaults to False. Optional.
        sheetNames (list[str]): Expects a list of strings, where each
            string is a name for one of the datasets. When used, there
            must be an equal number of string names in sheetName as
            there are datasets in the dataset parameter. Names provided
            in this parameter may be sanitized into acceptable Excel
            sheet names. Optional.

    Returns:
        str: An Excel-compatible XML-based workbook, with one worksheet
            per dataset.
    """
    print(showHeaders, dataset, nullsEmpty, sheetNames)


def toPyDataSet(dataset):
    """This function converts from a normal DataSet to a PyDataSet,
    which is a wrapper class which makes working with datasets more
    Python-esque.

    Args:
        dataset (Dataset): A DataSet object to convert into a PyDataSet.

    Returns:
        PyDataSet: The newly created PyDataSet.
    """
    print(dataset)
    return DatasetUtilities.PyDataSet()


def updateRow(dataset, rowIndex, changes):
    """Takes a dataset and returns a new dataset with a one row altered.

    Datasets are immutable, so it is important to realize that this
    function does not actually change the row in the argument dataset.
    You'll need to do something with the new dataset that this function
    creates to achieve something useful.

    To alter the row, this function takes a Python dictionary to
    represent the changes to make to the specified row. The keys in the
    dictionary are used to find the columns to alter.

    Args:
        dataset (Dataset): The starting dataset. Will not be modified
            (datasets are immutable), but acts as the basis for the
            returned dataset.
        rowIndex (int): The index of the row to update (starting at 0).
        changes (dict): A Dictionary of changes to make. They keys in
            the dictionary should match column names in the dataset, and
            their values will be used to update the row.

    Returns:
         BasicDataset: A new dataset with the values at the specified
            row updated according to the values in the dictionary.
    """
    print(dataset, rowIndex, changes)
    return BasicDataset()



"""Report Functions.

The following functions give you access to report details and the
ability to run reports.
"""

from __future__ import print_function

__all__ = [
    "executeAndDistribute",
    "executeReport",
    "getReportNamesAsDataset",
    "getReportNamesAsList",
]

from com.inductiveautomation.ignition.common import BasicDataset
from java.lang import IllegalArgumentException


def executeAndDistribute(
    path, project="project", parameters=None, action=None, actionSettings=None
):
    """Executes and distributes a report.

    Similar to scheduling a report to execute, except a schedule in not
    required to utilize this function. This is a great way to distribute
    the report on demand from a client.

    Args:
        path (str): The path to the existing report.
        project (str): The name of the project where the report is
            located. Optional in client scope.
        parameters (dict): An optional dictionary of parameter
            overrides, in the form name:value.
        action (str): The name of the distribution action to use.
        actionSettings (dict): An optional dictionary of settings
            particular to the action. Missing values will use the
            default value for that action.

    Raises:
        IllegalArgumentException: Thrown when any of the following
            occurs: If the file type is not recognized, path does not
            exist, project does not exist, or a key is not valid.
    """
    if project is not None:
        print(path, project, parameters, action, actionSettings)
    else:
        raise IllegalArgumentException()


def executeReport(path, project="project", parameters=None, fileType="pdf"):
    """Immediately executes an existing report and returns a byte[] of
    the output.

    Args:
        path (str): The path to the existing report.
        project (str): The name of the project where the report is
            located. Optional in client scope.
        parameters (dict): An optional dictionary of parameter
            overrides, in the form name:value. Optional.
        fileType (str): The file type the resulting byte array should
            represent. Defaults to "pdf". Not case-sensitive. Optional.

    Returns:
        object: A byte array of the resulting report.

    Raises:
        IllegalArgumentException: Thrown when any of the following
            occurs: If the file type is not recognized, path does not
            exist, project does not exist.
    """
    file_types = [
        "csv",
        "html",
        "jpeg",
        "pdf",
        "png",
        "rtf",
        "xls",
        "xlsx",
        "xml",
    ]
    if path is None or project is None or fileType not in file_types:
        raise IllegalArgumentException()
    print(path, project, parameters, fileType)


def getReportNamesAsDataset(project="project", includeReportName=True):
    """Gets a data of all reports for a project.

    This dataset is particularly suited for display in a Tree View
    component.

    Args:
        project (str): The name of the project where the reports are
            located. Optional in client scope.
        includeReportName (bool): When set to False, the end of Path
            does not include the report name. Default is True. Optional.

    Returns:
        BasicDataset: A dataset of report paths and names for the
            project. Returns an empty dataset if the project has no
            reports.

    Raises:
        IllegalArgumentException: Thrown when any of the following
            occurs: If the project name is omitted in the Gateway scope,
            project does not exist.
    """
    if project is None:
        raise IllegalArgumentException()

    print(project, includeReportName)
    return BasicDataset()


def getReportNamesAsList(project="project"):
    """Gets a list of all reports for a project.

    Args:
        project (str): The name of the project where the reports are
            located. Optional in client scope.

    Returns:
        list[str]: A list of report paths for the project. Returns an
            empty list if the project has no reports.

    Raises:
        IllegalArgumentException: Thrown when any of the following
            occurs: If the project name is omitted in the Gateway scope,
            project does not exist.
    """
    if project is None:
        raise IllegalArgumentException()

    return []



"""Provides a set of "lightweight" (all-Java language) components that,
to the maximum degree possible, work the same on all platforms.
"""

from __future__ import print_function

__all__ = [
    "JComponent",
    "JFrame",
    "JInternalFrame",
    "JLabel",
    "JOptionPane",
    "JPanel",
    "JPopupMenu",
    "JTextField",
]

from java.awt import Container, Frame
from javax.swing.text import JTextComponent


class JComponent(Container):
    """The base class for all Swing components except top-level
    containers.
    """

    pass


class JFrame(Frame):
    """An extended version of java.awt.Frame that adds support for the
    JFC/Swing component architecture.
    """

    pass


class JInternalFrame(JComponent):
    """A lightweight object that provides many of the features of a
    native frame, including dragging, closing, becoming an icon,
    resizing, title display, and support for a menu bar.
    """

    pass


class JLabel(JComponent):
    """A display area for a short text string or an image, or both."""

    def __init__(self, *args):
        """Creates a JLabel instance."""
        pass


class JOptionPane(JComponent):
    """JOptionPane makes it easy to pop up a standard dialog box that
    prompts users for a value or informs them of something. For
    information about using JOptionPane, see How to Make Dialogs, a
    section in The Java Tutorial.
    """

    # messageType.
    PLAIN_MESSAGE = -1
    ERROR_MESSAGE = 0
    INFORMATION_MESSAGE = 1
    WARNING_MESSAGE = 2
    QUESTION_MESSAGE = 3

    # optionType.
    DEFAULT_OPTION = -1
    YES_NO_OPTION = 0
    YES_NO_CANCEL_OPTION = 1
    OK_CANCEL_OPTION = 2

    # When one of the showXxxDialog methods returns an integer, the
    # possible values are:
    CLOSED_OPTION = -1
    OK_OPTION = 0
    YES_OPTION = 0
    NO_OPTION = 1
    CANCEL_OPTION = 2

    @staticmethod
    def showConfirmDialog(
        parentComponent,
        message,
        title=None,
        optionType=None,
        messageType=None,
        icon=None,
    ):
        """Asks a confirming question, like yes/no/cancel.

        Args:
            parentComponent (Component): Determines the Frame in which
                the dialog is displayed; if None, or if the
                parentComponent has no Frame, a default Frame is used.
            message (object): The object to display.
            title (str): The title string for the dialog. Optional.
            optionType (int): An int designating the options available
                on the dialog: YES_NO_OPTION, YES_NO_CANCEL_OPTION, or
                OK_CANCEL_OPTION. Optional.
            messageType (int): An integer designating the kind of
                message this is; primarily used to determine the icon
                from the pluggable Look and Feel: ERROR_MESSAGE,
                INFORMATION_MESSAGE, WARNING_MESSAGE, QUESTION_MESSAGE,
                or PLAIN_MESSAGE. Optional.
            icon (Icon): The icon to display in the dialog. Optional.

        Returns:
            int: An integer indicating the option selected by the user.
        """
        print(parentComponent, message, title, optionType, messageType, icon)
        return JOptionPane.YES_OPTION

    @staticmethod
    def showInputDialog(
        parentComponent,
        message,
        title=None,
        messageType=None,
        icon=None,
        selectionValues=None,
        initialSelectionValue=None,
    ):
        """Prompt for some input.

        Args:
            parentComponent (Component): Determines the Frame in which
                the dialog is displayed; if None, or if the
                parentComponent has no Frame, a default Frame is used.
            message (object): The object to display.
            title (str): The title string for the dialog. Optional.
            messageType (int): The type of message to be displayed:
                ERROR_MESSAGE, INFORMATION_MESSAGE, WARNING_MESSAGE,
                QUESTION_MESSAGE, or PLAIN_MESSAGE. Optional.
            icon (Icon): The icon to display in the dialog. Optional.
            selectionValues (object): An array of Objects that gives the
                possible selections. Optional.
            initialSelectionValue (object): The value used to initialize
                the input field. Optional.

        Returns:
            object: An integer indicating the option selected by the
                user.
        """
        print(
            parentComponent,
            message,
            title,
            messageType,
            icon,
            selectionValues,
            initialSelectionValue,
        )
        return "Input"

    @staticmethod
    def showMessageDialog(
        parentComponent, message, title=None, messageType=None, icon=None
    ):
        """Tell the user about something that has happened.

        Args:
            parentComponent (Component): Determines the Frame in which
                the dialog is displayed; if None, or if the
                parentComponent has no Frame, a default Frame is used.
            message (object): The object to display.
            title (str): The title string for the dialog. Optional.
            messageType (int): The type of message to be displayed:
                ERROR_MESSAGE, INFORMATION_MESSAGE, WARNING_MESSAGE,
                QUESTION_MESSAGE, or PLAIN_MESSAGE. Optional.
            icon (Icon): The icon to display in the dialog. Optional.
        """
        print(parentComponent, message, title, messageType, icon)

    @staticmethod
    def showOptionDialog(
        parentComponent,
        message,
        title=None,
        optionType=None,
        messageType=None,
        icon=None,
        options=None,
        initialValue=None,
    ):
        """The Grand Unification of the above three.

        Args:
            parentComponent (Component): Determines the Frame in which
                the dialog is displayed; if None, or if the
                parentComponent has no Frame, a default Frame is used.
            message (object): The object to display.
            title (str): The title string for the dialog. Optional.
            optionType (int): An integer designating the options
                available on the dialog: DEFAULT_OPTION, YES_NO_OPTION,
                YES_NO_CANCEL_OPTION, or OK_CANCEL_OPTION. Optional.
            messageType (int): An integer designating the kind of
                message this is; primarily used to determine the icon
                from the pluggable Look and Feel: ERROR_MESSAGE,
                INFORMATION_MESSAGE, WARNING_MESSAGE, QUESTION_MESSAGE,
                or PLAIN_MESSAGE. Optional.
            icon (Icon): The icon to display in the dialog. Optional.
            options (list): An array of objects indicating the possible
                choices the user can make; if the objects are
                components, they are rendered properly; non-String
                objects are rendered using their toString methods; if
                this parameter is null, the options are determined by
                the Look and Feel. Optional.
            initialValue (object): The object that represents the
                default selection for the dialog; only meaningful if
                options is used; can be null. Optional.

        Returns:
            int: An integer indicating the option chosen by the user, or
                CLOSED_OPTION if the user closed the dialog.
        """
        print(
            parentComponent,
            message,
            title,
            optionType,
            messageType,
            icon,
            options,
            initialValue,
        )
        return JOptionPane.YES_OPTION


class JPanel(JComponent):
    """JPanel is a generic lightweight container."""

    pass


class JPopupMenu(JComponent):
    """An implementation of a popup menu -- a small window that pops up
    and displays a series of choices. A JPopupMenu is used for the menu
    that appears when the user selects an item on the menu bar. It is
    also used for "pull-right" menu that appears when the selects a menu
    item that activates it. Finally, a JPopupMenu can also be used
    anywhere else you want a menu to appear. For example, when the user
    right-clicks in a specified area.
    """

    pass


class JTextField(JTextComponent):
    """JTextField is a lightweight component that allows the editing of
    a single line of text.
    """

    def __init__(self, *args):
        """Constructor."""
        pass



__all__ = ["Result", "Results"]

from java.lang import Object


class Result(object):
    def getDisplayPath(self):
        raise NotImplementedError

    def getPath(self):
        raise NotImplementedError

    def getType(self):
        raise NotImplementedError

    def hasChildren(self):
        raise NotImplementedError


class Results(Object):
    """The results of a browse operation.

    May only represent a partial result set, which can be determined by
    comparing the Total Available Size to the Returned Size. If there is
    a mismatch, the continuation point should be non-null and can be
    used in constructing the subsequent BrowseFilter to continue the
    browse.
    """

    def error(self, result):
        pass

    def getContinuationPoint(self):
        pass

    def getResultQuality(self):
        pass

    def getResults(self):
        pass

    def getReturnedSize(self):
        pass

    def getTotalAvailableSize(self):
        pass

    def of(self, arg):
        pass

    def setContinuationPoint(self, continuationPoint):
        pass

    def setResultQuality(self, value):
        pass

    def setResults(self, results):
        pass

    def setTotalAvailableResults(self, totalAvailableResults):
        pass



__all__ = ["ShelvedPath"]

from java.lang import Object


class ShelvedPath(Object):
    """This class provides information about a path that has been
    "shelved", such as when it was shelved, and by whom, and the actual
    path.
    """

    def __init__(self, path=None, user=None, expiration=None):
        self.path = path
        self.user = user
        self.expiration = expiration

    def getExpiration(self):
        pass

    def getHitCount(self):
        pass

    def getPath(self):
        pass

    def getShelveTime(self):
        pass

    def getUser(self):
        pass

    def incrementHitCount(self):
        pass

    def isExpired(self):
        pass
